# -*- coding: utf-8 -*-
import re
from abjad.tools import markuptools
from abjad.tools import systemtools
from abjad.tools.abctools import AbjadValueObject


class RomanNumeral(AbjadValueObject):
    '''A functions in tonal harmony: I, I6, I64, V, V7, V43, V42,
    bII, bII6, etc., also i, i6, i64, v, v7, etc.

    Value object that can not be changed after instantiation.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_extent',
        '_inversion',
        '_quality',
        '_scale_degree',
        '_suspension',
        )

    _figured_bass_string_to_extent = {
        '': 5,
        '6': 5,
        '6/4': 5,
        '7': 7,
        '6/5': 7,
        '4/3': 7,
        '4/2': 7,
        }

    _figured_bass_string_to_inversion = {
        '': 0,
        '6': 1,
        '6/4': 2,
        '7': 0,
        '6/5': 1,
        '4/3': 2,
        '4/2': 3,
        }

    _symbolic_string_regex = re.compile(
        r'([#|b]*)([i|I|v|V]+)([M|m|o|@|+]?)(.*)')

    ### INITIALIZER ###

    def __init__(self, *arguments):
        if len(arguments) == 1 and isinstance(arguments[0], type(self)):
            scale_degree, quality, extent, inversion, suspension = \
                self._initialize_by_reference(arguments[0])
        elif len(arguments) == 1 and isinstance(arguments[0], str):
            scale_degree, quality, extent, inversion, suspension = \
                self._initialize_by_symbolic_string(arguments[0])
        elif len(arguments) == 4:
            scale_degree, quality, extent, inversion, suspension = \
                self._initialize_by_scale_degree_quality_extent_and_inversion(*arguments)
        elif len(arguments) == 5:
            scale_degree, quality, extent, inversion, suspension = \
                self._initialize_with_suspension(*arguments)
        elif len(arguments) == 0:
            scale_degree, quality, extent, inversion, suspension = \
                self._initialize_by_symbolic_string('I')
        else:
            message = 'can not initialize tonal function: {!r}.'
            message = message.format(arguments)
            raise ValueError(message)
        self._scale_degree = scale_degree
        self._quality = quality
        self._extent = extent
        self._inversion = inversion
        self._suspension = suspension

    ### SPECIAL METHODS ###

    def __eq__(self, arg):
        r'''Is true when `arg` is a roman numeral with scale degree, quality,
        extent, inversion and suspension equal to those of this roman numeral.
        Otherwise false.

        Returns true or false.
        '''
        if isinstance(arg, type(self)):
            if self.scale_degree == arg.scale_degree:
                if self.quality == arg.quality:
                    if self.extent == arg.extent:
                        if self.inversion == arg.inversion:
                            if self.suspension == arg.suspension:
                                return True
        return False

    def __hash__(self):
        r'''Hashes roman numeral.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(RomanNumeral, self).__hash__()

    def __ne__(self, arg):
        r'''Is true when roman numeral does not equal `arg`. Otherwise false.

        Returns true or false.
        '''
        return not self == arg

    def __repr__(self):
        r'''Gets interpreter representation of `arg`.

        Returns string.
        '''
        return self._format_string

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        return systemtools.FormatSpecification(
            client=self,
            storage_format_is_indented=False,
            storage_format_args_values=[self.symbolic_string],
            )

    def _get_quality_name(self, uppercase, quality_string, extent):
        if quality_string == 'o':
            return 'diminished'
        elif quality_string == '@':
            return 'half diminished'
        elif quality_string == '+':
            return 'augmented'
        elif quality_string == 'M':
            return 'major'
        elif quality_string == 'm':
            return 'minor'
        elif extent == 5:
            if quality_string == '' and uppercase:
                return 'major'
            elif quality_string == '' and not uppercase:
                return 'minor'
            else:
                raise ValueError
        elif extent == 7:
            if quality_string == '' and uppercase:
                return 'dominant'
            elif quality_string == '' and not uppercase:
                return 'minor'
            else:
                raise ValueError
        else:
            raise ValueError

    def _initialize_by_reference(self, tonal_function):
        arguments = (tonal_function.scale_degree, tonal_function.quality,
            tonal_function.extent, tonal_function.inversion)
        return self._initialize_by_scale_degree_quality_extent_and_inversion(
            self, *arguments)

    def _initialize_by_scale_degree_quality_extent_and_inversion(self, *arguments):
        from abjad.tools import tonalanalysistools
        scale_degree, quality, extent, inversion = arguments
        scale_degree = tonalanalysistools.ScaleDegree(scale_degree)
        quality = tonalanalysistools.ChordQuality(quality)
        extent = tonalanalysistools.ChordExtent(extent)
        inversion = tonalanalysistools.ChordInversion(inversion)
        suspension = tonalanalysistools.ChordSuspension()
        return scale_degree, quality, extent, inversion, suspension

    def _initialize_by_symbolic_string(self, symbolic_string):
        from abjad.tools import tonalanalysistools
        groups = self._symbolic_string_regex.match(symbolic_string).groups()
        accidental, roman_numeral, quality, figured_bass = groups
        scale_degree = tonalanalysistools.ScaleDegree(accidental + roman_numeral)
        figured_bass_parts = figured_bass.split('/')
        naive_figured_bass = [x for x in figured_bass_parts if '-' not in x]
        naive_figured_bass = '/'.join(naive_figured_bass)
        extent = self._figured_bass_string_to_extent[naive_figured_bass]
        extent = tonalanalysistools.ChordExtent(extent)
        uppercase = roman_numeral == roman_numeral.upper()
        quality = self._get_quality_name(uppercase, quality, extent.number)
        quality = tonalanalysistools.ChordQuality(quality)
        inversion = self._figured_bass_string_to_inversion[naive_figured_bass]
        inversion = tonalanalysistools.ChordInversion(inversion)
        suspension = [x for x in figured_bass_parts if '-' in x]
        if not suspension:
            suspension = tonalanalysistools.ChordSuspension()
        elif 1 < len(suspension):
            message = 'no multiple suspensions yet.'
            raise NotImplementedError(message)
        else:
            suspension = tonalanalysistools.ChordSuspension(suspension[0])
        return scale_degree, quality, extent, inversion, suspension

    def _initialize_with_suspension(self, *arguments):
        from abjad.tools import tonalanalysistools
        scale_degree, quality, extent, inversion, suspension = \
            self._initialize_by_scale_degree_quality_extent_and_inversion(*arguments[:-1])
        suspension = arguments[-1]
        suspension = tonalanalysistools.ChordSuspension(suspension)
        return scale_degree, quality, extent, inversion, suspension

    ### PRIVATE PROPERTIES ###

    @property
    def _accidental_name(self):
        accidental = self.scale_degree.accidental
        if accidental.is_adjusted:
            return accidental.name.title()
        return ''

    @property
    def _figured_bass_digits(self):
        characters = self._figured_bass_string
        if characters:
            characters = characters.split('/')
            digits = [int(x) for x in characters]
            return tuple(digits)
        return ()

    @property
    def _figured_bass_string(self):
        return self.inversion.extent_to_figured_bass_string(self.extent.number)

    @property
    def _format_string(self):
        result = []
        result.append(self._accidental_name)
        result.append(self._roman_numeral_string)
        result.append(self.quality.quality_string.title())
        result.append(self.extent.name.title())
        result.append('In')
        result.append(self.inversion.title)
        if not self.suspension.is_empty:
            result.append('With')
            result.append(self.suspension.title_string)
        return ''.join(result)

    @property
    def _quality_symbolic_string(self):
        from abjad.tools import tonalanalysistools
        if self.extent == tonalanalysistools.ChordExtent(5):
            if self.quality == tonalanalysistools.ChordQuality('diminished'):
                return 'o'
            elif self.quality == tonalanalysistools.ChordQuality('augmented'):
                return '+'
            else:
                return ''
        elif self.extent == tonalanalysistools.ChordExtent(7):
            if self.quality == tonalanalysistools.ChordQuality('dominant'):
                return ''
            elif self.quality == tonalanalysistools.ChordQuality('major'):
                return 'M'
            elif self.quality == \
                tonalanalysistools.ChordQuality('diminished'):
                return 'o'
            elif self.quality == \
                tonalanalysistools.ChordQuality('half diminished'):
                return '@'
            elif self.quality == tonalanalysistools.ChordQuality('augmented'):
                return '+'
            else:
                return ''
        else:
            raise NotImplementedError

    @property
    def _roman_numeral_string(self):
        roman_numeral_string = self.scale_degree.roman_numeral_string
        if not self.quality.is_uppercase:
            roman_numeral_string = roman_numeral_string.lower()
        return roman_numeral_string

    ### PUBLIC PROPERTIES ###

    @property
    def bass_scale_degree(self):
        r'''Base scale degree of roman numeral.

        Returns scale degree.
        '''
        from abjad.tools import tonalanalysistools
        root_scale_degree = self.root_scale_degree.number
        bass_scale_degree = root_scale_degree - 1
        bass_scale_degree += 2 * self.inversion.number
        bass_scale_degree %= 7
        bass_scale_degree += 1
        bass_scale_degree = tonalanalysistools.ScaleDegree(bass_scale_degree)
        return bass_scale_degree

    @property
    def extent(self):
        r'''Extend of roman numeral.

        Returns extent.
        '''
        return self._extent

    @property
    def figured_bass_string(self):
        r'''Figured bass string of roman numeral.

        Returns string.
        '''
        digits = self._figured_bass_digits
        if self.suspension.is_empty:
            return '/'.join([str(x) for x in digits])
        suspension_pair = self.suspension.figured_bass_pair
        figured_bass_list = []
        for n in range(9, 1, -1):
            if n == suspension_pair[0]:
                figured_bass_list.append(str(self.suspension))
            elif n == suspension_pair[1]:
                pass
            elif n in digits:
                figured_bass_list.append(str(n))
        figured_bass_string = '/'.join(figured_bass_list)
        return figured_bass_string

    @property
    def inversion(self):
        r'''Inversion of roman numeral.

        Returns nonnegative integer.
        '''
        return self._inversion

    @property
    def markup(self):
        r'''Markup of roman numeral.

        Returns markup.
        '''
        symbolic_string = self.symbolic_string
        symbolic_string = symbolic_string.replace('#', r'\sharp ')
        return markuptools.Markup(symbolic_string, Down)

    @property
    def quality(self):
        r'''Quality of roman numeral.

        Returns chord quality.
        '''
        return self._quality

    @property
    def root_scale_degree(self):
        r'''Root scale degree.

        Returns scale degree.
        '''
        return self._scale_degree

    # TODO: deprecate scale_degree in favor of root_scale_degree
    @property
    def scale_degree(self):
        r'''Scale degree of roman numeral.

        Returns scale degree.
        '''
        return self._scale_degree

    @property
    def suspension(self):
        r'''Suspension of roman numeral.

        Returns suspension.
        '''
        return self._suspension

    @property
    def symbolic_string(self):
        r'''Symbolic string of roman numeral.

        Returns string.
        '''
        result = ''
        result += self.scale_degree.accidental.symbolic_string
        result += self._roman_numeral_string
        result += self._quality_symbolic_string
        result += self.figured_bass_string
        return result



# -*- coding: utf-8 -*-
from abjad import *
import pytest


def test_selectiontools_Parentage_logical_voice_01():
    r'''An anonymous staff and its contained unvoiced leaves share
    the same signature.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")

    containment = inspect_(staff).get_parentage().logical_voice
    for component in iterate(staff).by_class():
        assert inspect_(component).get_parentage().logical_voice == containment


def test_selectiontools_Parentage_logical_voice_02():
    r'''A named staff and its contained unvoiced leaves share
    the same signature.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    staff.name = 'foo'

    containment = inspect_(staff).get_parentage().logical_voice
    for component in iterate(staff).by_class():
        assert inspect_(component).get_parentage().logical_voice == containment

def test_selectiontools_Parentage_logical_voice_03():
    r'''Leaves inside equally named sequential voices inside a staff
    share the same signature.
    '''

    staff = Staff(Voice("c'8 d'8 e'8 f'8") * 2)
    staff[0].name = 'foo'
    staff[1].name = 'foo'

    containment = inspect_(staff[0][0]).get_parentage().logical_voice
    for leaf in iterate(staff).by_leaf():
        assert inspect_(leaf).get_parentage().logical_voice == containment


def test_selectiontools_Parentage_logical_voice_04():
    r'''Returns logical voice giving the root and
    first voice, staff and score in the parentage of component.
    '''

    voice = Voice(
        r'''
        c'8
        d'8
        <<
            \new Voice {
                e'8
                f'8
            }
            \new Voice {
                g'8
                a'8
            }
        >>
        b'8
        c''8
        '''
        )

    override(voice).note_head.color = 'red'

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice \with {
            \override NoteHead.color = #red
        } {
            c'8
            d'8
            <<
                \new Voice {
                    e'8
                    f'8
                }
                \new Voice {
                    g'8
                    a'8
                }
            >>
            b'8
            c''8
        }
        '''
        )

    signatures = [inspect_(leaf).get_parentage().logical_voice
        for leaf in iterate(voice).by_leaf()]

    assert signatures[0] == signatures[1]
    assert signatures[0] != signatures[2]
    assert signatures[0] != signatures[4]
    assert signatures[0] == signatures[6]

    assert signatures[2] == signatures[3]
    assert signatures[2] != signatures[4]


def test_selectiontools_Parentage_logical_voice_05():
    r'''Returns logical voice giving the root and
    first voice, staff and score in parentage of component.
    '''

    voice = Voice(
        r'''
        c'8
        d'8
        <<
            \context Voice = "foo" {
                e'8
                f'8
            }
            \new Voice {
                g'8
                a'8
            }
        >>
        b'8
        c''8
        '''
        )

    override(voice).note_head.color = 'red'
    voice.name = 'foo'

    assert format(voice) == stringtools.normalize(
        r'''
        \context Voice = "foo" \with {
            \override NoteHead.color = #red
        } {
            c'8
            d'8
            <<
                \context Voice = "foo" {
                    e'8
                    f'8
                }
                \new Voice {
                    g'8
                    a'8
                }
            >>
            b'8
            c''8
        }
        '''
        )

    signatures = [
        inspect_(leaf).get_parentage().logical_voice
        for leaf in iterate(voice).by_leaf()
        ]

    signatures[0] == signatures[1]
    signatures[0] == signatures[2]
    signatures[0] != signatures[4]
    signatures[0] == signatures[6]

    signatures[2] == signatures[0]
    signatures[2] == signatures[3]
    signatures[2] == signatures[4]
    signatures[2] == signatures[6]

    signatures[4] != signatures[0]
    signatures[4] != signatures[2]
    signatures[4] == signatures[5]
    signatures[4] == signatures[6]


def test_selectiontools_Parentage_logical_voice_06():
    r'''Returns logical voice giving the root and
    first voice, staff and score in parentage of component.
    '''

    container = Container([
        Staff([Voice("c'8 d'8")]),
        Staff([Voice("e'8 f'8")]),
        ])
    container[0].name = 'staff1'
    container[1].name = 'staff2'
    container[0][0].name = 'voicefoo'
    container[1][0].name = 'voicefoo'
    beam = Beam()
    leaves = select(container).by_leaf()
    statement = 'attach(beam, leaves)'
    assert pytest.raises(Exception, statement)
    beam = Beam()
    attach(beam, leaves[:2])
    beam = Beam()
    attach(beam, leaves[2:])

    assert format(container) == stringtools.normalize(
        r'''
        {
            \context Staff = "staff1" {
                \context Voice = "voicefoo" {
                    c'8 [
                    d'8 ]
                }
            }
            \context Staff = "staff2" {
                \context Voice = "voicefoo" {
                    e'8 [
                    f'8 ]
                }
            }
        }
        '''
        )

    signatures = [inspect_(leaf).get_parentage().logical_voice
        for leaf in leaves]

    signatures[0] == signatures[1]
    signatures[0] != signatures[2]

    signatures[2] != signatures[2]
    signatures[2] == signatures[3]


def test_selectiontools_Parentage_logical_voice_07():
    r'''Returns logical voice giving the root and
    first voice, staff and score in parentage of component.
    '''

    container = Container(
        r'''
        c'8
        <<
            \context Voice = "alto" {
                d'8
            }
            \context Voice = "soprano" {
                e'8
            }
        >>
        {
            \context Voice = "alto" {
                f'8
            }
            \context Voice = "soprano" {
                g'8
            }
        }
        a'8
        '''
        )

    override(container[1][1]).note_head.color = 'red'
    override(container[2][1]).note_head.color = 'red'

    assert format(container) == stringtools.normalize(
        r'''
        {
            c'8
            <<
                \context Voice = "alto" {
                    d'8
                }
                \context Voice = "soprano" \with {
                    \override NoteHead.color = #red
                } {
                    e'8
                }
            >>
            {
                \context Voice = "alto" {
                    f'8
                }
                \context Voice = "soprano" \with {
                    \override NoteHead.color = #red
                } {
                    g'8
                }
            }
            a'8
        }
        '''
        )

    signatures = [
        inspect_(leaf).get_parentage().logical_voice
        for leaf in iterate(container).by_leaf()
        ]

    signatures[0] != signatures[1]
    signatures[0] != signatures[2]
    signatures[0] != signatures[3]
    signatures[0] != signatures[4]
    signatures[0] == signatures[5]

    signatures[1] != signatures[0]
    signatures[1] != signatures[2]
    signatures[1] == signatures[3]
    signatures[1] != signatures[4]
    signatures[1] != signatures[5]

    signatures[2] != signatures[0]
    signatures[2] != signatures[1]
    signatures[2] != signatures[3]
    signatures[2] == signatures[4]
    signatures[2] != signatures[5]


def test_selectiontools_Parentage_logical_voice_08():
    r'''Unicorporated leaves carry equivalent containment signatures.
    '''

    note_1 = Note(0, (1, 8))
    note_2 = Note(0, (1, 8))

    signature_1 = inspect_(note_1).get_parentage().logical_voice
    signature_2 = inspect_(note_2).get_parentage().logical_voice
    assert signature_1 == signature_2


def test_selectiontools_Parentage_logical_voice_09():
    r'''Notes appear in the same logical voice.
    '''

    staff_1 = Staff([Voice([Note(0, (1, 8))])])
    staff_1.name = 'staff'
    staff_1[0].name = 'voice'

    staff_2 = Staff([Voice([Note(0, (1, 8))])])
    staff_2.name = 'staff'
    staff_2[0].name = 'voice'

    staff_1_leaf_signature = inspect_(
        staff_1[0][0]).get_parentage().logical_voice
    staff_2_leaf_signature = inspect_(
        staff_2[0][0]).get_parentage().logical_voice
    assert staff_1_leaf_signature == staff_2_leaf_signature


def test_selectiontools_Parentage_logical_voice_10():
    r'''Measure and leaves must carry same logical voice signature.
    '''

    staff = Staff(r'''
        {
            \time 2/8
            c'8
            d'8
        }
        e'8
        f'8
        ''')

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            e'8
            f'8
        }
        '''
        )

    assert inspect_(staff[0]).get_parentage().logical_voice == \
        inspect_(staff[-1]).get_parentage().logical_voice
    assert inspect_(staff[0]).get_parentage().logical_voice == \
        inspect_(staff[0][0]).get_parentage().logical_voice
    assert inspect_(staff[0][0]).get_parentage().logical_voice == \
        inspect_(staff[-1]).get_parentage().logical_voice


def test_selectiontools_Parentage_logical_voice_11():
    r'''Leaves inside different staves have different logical voice
    signatures, even when the staves have the same name.
    '''

    container = Container(2 * Staff("c'8 c'8"))
    container[0].name = container[1].name = 'staff'

    assert format(container) == stringtools.normalize(
        r'''
        {
            \context Staff = "staff" {
                c'8
                c'8
            }
            \context Staff = "staff" {
                c'8
                c'8
            }
        }
        '''
        )

    leaves = select(container).by_leaf()
    assert inspect_(leaves[0]).get_parentage().logical_voice == \
        inspect_(leaves[1]).get_parentage().logical_voice
    assert inspect_(leaves[0]).get_parentage().logical_voice != \
        inspect_(leaves[2]).get_parentage().logical_voice
    assert inspect_(leaves[2]).get_parentage().logical_voice == \
        inspect_(leaves[3]).get_parentage().logical_voice
    assert inspect_(leaves[2]).get_parentage().logical_voice != \
        inspect_(leaves[0]).get_parentage().logical_voice



# -*- coding: utf-8 -*-
from abjad.tools import mathtools
from abjad.tools.pitchtools.Segment import Segment
from abjad.tools.topleveltools import new


class IntervalClassSegment(Segment):
    r'''Interval-class segment.

    ..  container:: example

        An interval-class segment:

        ::

            >>> intervals = 'm2 M10 -aug4 P5'
            >>> pitchtools.IntervalClassSegment(intervals)
            IntervalClassSegment(['+m2', '+M3', '-aug4', '+P5'])

    ..  container:: example

        Another interval-class segment:

        ::

            >>> intervals = 'P4 P5 P11 P12'
            >>> pitchtools.IntervalClassSegment(intervals)
            IntervalClassSegment(['+P4', '+P5', '+P4', '+P5'])

    Returns interval-class segment.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### PRIVATE PROPERTIES ###

    @property
    def _named_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.NamedIntervalClass

    @property
    def _numbered_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.NumberedIntervalClass

    @property
    def _parent_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.IntervalClass

    ### PUBLIC PROPERTIES ###

    @property
    def is_tertian(self):
        r'''Is true when all named interval-classes in segment are tertian.
        Otherwise false:

        ::

            >>> interval_class_segment = pitchtools.IntervalClassSegment(
            ...     items=[('major', 3), ('minor', 6), ('major', 6)],
            ...     item_class=pitchtools.NamedIntervalClass,
            ...     )
            >>> interval_class_segment.is_tertian
            True

        Returns true or false.
        '''
        from abjad.tools import pitchtools
        inversion_equivalent_interval_class_segment = new(
            self,
            item_class=pitchtools.NamedInversionEquivalentIntervalClass,
            )
        for interval in inversion_equivalent_interval_class_segment:
            if not interval.number == 3:
                return False
        return True

    ### PUBLIC METHODS ###

    @classmethod
    def from_selection(class_, selection, item_class=None):
        r'''Initialize interval-class segment from component selection:

        ::

            >>> staff_1 = Staff("c'4 <d' fs' a'>4 b2")
            >>> staff_2 = Staff("c4. r8 g2")
            >>> selection = select((staff_1, staff_2))
            >>> pitchtools.IntervalClassSegment.from_selection(selection)
            IntervalClassSegment(['-M2', '-M3', '-m3', '+m7', '+M7', '-P5'])

        Returns interval-class segment.
        '''
        from abjad.tools import pitchtools
        pitch_segment = pitchtools.PitchSegment.from_selection(selection)
        pitches = [_ for _ in pitch_segment]
        intervals = mathtools.difference_series(pitches)
        return class_(
            items=intervals,
            item_class=item_class,
            )

    def has_duplicates(self):
        r'''Is true when segment contains duplicates. Otherwise false.

        ::

            >>> intervals = 'm2 M3 -aug4 m2 P5'
            >>> segment = pitchtools.IntervalClassSegment(intervals)
            >>> segment.has_duplicates()
            True

        ::

            >>> intervals = 'M3 -aug4 m2 P5'
            >>> segment = pitchtools.IntervalClassSegment(intervals)
            >>> segment.has_duplicates()
            False

        Returns true or false.
        '''
        from abjad.tools import pitchtools
        return len(pitchtools.IntervalClassSet(
            self, item_class=self.item_class)) < len(self)



# -*- coding: utf-8 -*-


def all_are_positive_integers(argument):
    '''Is true when `argument` is an iterable collection of positive integers.
    Otherwise false.

    ..  container:: example

        ::

            >>> mathtools.all_are_positive_integers([1, 2, 3, 99])
            True

        ::

            >>> mathtools.all_are_positive_integers(17)
            False

    Returns true or false.
    '''
    from abjad.tools import mathtools
    try:
        return all(mathtools.is_positive_integer(_) for _ in argument)
    except TypeError:
        return False



# -*- coding: utf-8 -*-
from abjad.tools import datastructuretools
from abjad.tools.documentationtools.ReSTDirective import ReSTDirective


class ReSTAutosummaryDirective(ReSTDirective):
    r'''A ReST Autosummary directive.

    ::

        >>> toc = documentationtools.ReSTAutosummaryDirective()
        >>> for item in ['foo.Foo', 'bar.Bar', 'baz.Baz']:
        ...     toc.append(documentationtools.ReSTAutosummaryItem(text=item))
        ...
        >>> toc
        ReSTAutosummaryDirective(
            children=(
                ReSTAutosummaryItem(
                    text='foo.Foo'
                    ),
                ReSTAutosummaryItem(
                    text='bar.Bar'
                    ),
                ReSTAutosummaryItem(
                    text='baz.Baz'
                    ),
                ),
            directive='autosummary'
            )

    ::

        >>> print(toc.rest_format)
        .. autosummary::
        <BLANKLINE>
           foo.Foo
           bar.Bar
           baz.Baz

    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'reStructuredText'

    ### SPECIAL METHODS ###

    def __setitem__(self, i, argument):
        r'''Sets item `i` to `argument`.

        Returns none.
        '''
        from abjad.tools import documentationtools
        newexpr = []
        for x in argument:
            if isinstance(x, str):
                newexpr.append(documentationtools.ReSTAutosummaryItem(text=x))
            else:
                newexpr.append(x)
        datastructuretools.TreeContainer.__setitem__(self, i, newexpr)

    ### PRIVATE PROPERTIES ###

    @property
    def _children_rest_format_contributions(self):
        result = ['']
        for child in self.children:
            contribution = child._rest_format_contributions
            for x in contribution:
                if x:
                    result.append('   ' + x)
                else:
                    result.append(x)
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def directive(self):
        r'''Directive of ReST autosummary diretive.

        Returns ``'autosummary'``.
        '''
        return 'autosummary'

    @property
    def node_class(self):
        r'''Node class of ReST autosummary directive.
        '''
        from abjad.tools import documentationtools
        return (
            documentationtools.ReSTAutosummaryItem,
            )



# -*- coding: utf-8 -*-
import collections
import copy
import itertools
import numbers
from abjad.tools import durationtools
from abjad.tools import indicatortools
from abjad.tools import mathtools
from abjad.tools import scoretools
from abjad.tools import sequencetools
from abjad.tools import systemtools
from abjad.tools.abctools import AbjadObject


class QEventSequence(AbjadObject):
    r'''A well-formed sequence of q-events.

    Contains only pitched q-events and silent q-events, and terminates with a
    single terminal q-event.

    A q-event sequence is the primary input to the quantizer.

    A q-event sequence provides a number of convenience functions to
    assist with instantiating new sequences:

    ::

        >>> durations = (1000, -500, 1250, -500, 750)

    ::

        >>> sequence = \
        ...     quantizationtools.QEventSequence.from_millisecond_durations(
        ...     durations)

    ::

        >>> for q_event in sequence:
        ...     print(format(q_event, 'storage'))
        ...
        quantizationtools.PitchedQEvent(
            offset=durationtools.Offset(0, 1),
            pitches=(
                pitchtools.NamedPitch("c'"),
                ),
            )
        quantizationtools.SilentQEvent(
            offset=durationtools.Offset(1000, 1),
            )
        quantizationtools.PitchedQEvent(
            offset=durationtools.Offset(1500, 1),
            pitches=(
                pitchtools.NamedPitch("c'"),
                ),
            )
        quantizationtools.SilentQEvent(
            offset=durationtools.Offset(2750, 1),
            )
        quantizationtools.PitchedQEvent(
            offset=durationtools.Offset(3250, 1),
            pitches=(
                pitchtools.NamedPitch("c'"),
                ),
            )
        quantizationtools.TerminalQEvent(
            offset=durationtools.Offset(4000, 1),
            )

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_sequence',
        )

    ### INITIALIZER ###

    def __init__(self, sequence=None):
        from abjad.tools import quantizationtools
        q_event_classes = (
            quantizationtools.PitchedQEvent,
            quantizationtools.SilentQEvent,
            )
        #sequence = sequence or []
        if sequence is None:
            self._sequence = ()
            return
        else:
            assert 1 < len(sequence)
            assert all(isinstance(q_event, q_event_classes)
                for q_event in sequence[:-1])
            assert isinstance(sequence[-1], quantizationtools.TerminalQEvent)
            offsets = [x.offset for x in sequence]
            offsets = sequencetools.Sequence(offsets)
            assert offsets.is_increasing(strict=False)
            assert 0 <= sequence[0].offset
            self._sequence = tuple(sequence)

    ### SPECIAL METHODS ###

    def __contains__(self, argument):
        r'''Is true when q-event sequence contains `argument`. Otherwise false.

        Returns true or false.
        '''
        return argument in self._sequence

    def __eq__(self, argument):
        r'''Is true when q-event sequence equals `argument`. Otherwise false.

        Returns true or false.
        '''
        if type(self) == type(argument):
            if self.sequence == argument.sequence:
                return True
        return False

    def __format__(self, format_specification=''):
        r'''Formats q-event sequence.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

            >>> print(format(sequence))
            quantizationtools.QEventSequence(
                (
                    quantizationtools.PitchedQEvent(
                        offset=durationtools.Offset(0, 1),
                        pitches=(
                            pitchtools.NamedPitch("c'"),
                            ),
                        ),
                    quantizationtools.SilentQEvent(
                        offset=durationtools.Offset(1000, 1),
                        ),
                    quantizationtools.PitchedQEvent(
                        offset=durationtools.Offset(1500, 1),
                        pitches=(
                            pitchtools.NamedPitch("c'"),
                            ),
                        ),
                    quantizationtools.SilentQEvent(
                        offset=durationtools.Offset(2750, 1),
                        ),
                    quantizationtools.PitchedQEvent(
                        offset=durationtools.Offset(3250, 1),
                        pitches=(
                            pitchtools.NamedPitch("c'"),
                            ),
                        ),
                    quantizationtools.TerminalQEvent(
                        offset=durationtools.Offset(4000, 1),
                        ),
                    )
                )

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatAgent(self).get_storage_format()
        return str(self)

    def __getitem__(self, argument):
        r'''Gets item or slice identified by `argument`.

        Returns item or slice.
        '''
        return self._sequence.__getitem__(argument)

    def __hash__(self):
        r'''Hashes q-event sequence.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(QEventSequence, self).__hash__()

    def __iter__(self):
        r'''Iterates q-event sequence.

        Yields items.
        '''
        for x in self._sequence:
            yield x

    def __len__(self):
        r'''Length of q-event sequence.

        Returns nonnegative integer.
        '''
        return len(self._sequence)

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        values = []
        if self.sequence:
            values.append(self.sequence)
        return systemtools.FormatSpecification(
            client=self,
            storage_format_args_values=values,
            storage_format_kwargs_names=[],
            )

    ### PUBLIC METHODS ###

    @classmethod
    def from_millisecond_durations(cls, milliseconds, fuse_silences=False):
        r'''Convert a sequence of millisecond durations ``durations`` into
        a ``QEventSequence``:

        ::

            >>> durations = [-250, 500, -1000, 1250, -1000]

        ::

            >>> sequence = \
            ...     quantizationtools.QEventSequence.from_millisecond_durations(
            ...     durations)

        ::

            >>> for q_event in sequence:
            ...     print(format(q_event, 'storage'))
            ...
            quantizationtools.SilentQEvent(
                offset=durationtools.Offset(0, 1),
                )
            quantizationtools.PitchedQEvent(
                offset=durationtools.Offset(250, 1),
                pitches=(
                    pitchtools.NamedPitch("c'"),
                    ),
                )
            quantizationtools.SilentQEvent(
                offset=durationtools.Offset(750, 1),
                )
            quantizationtools.PitchedQEvent(
                offset=durationtools.Offset(1750, 1),
                pitches=(
                    pitchtools.NamedPitch("c'"),
                    ),
                )
            quantizationtools.SilentQEvent(
                offset=durationtools.Offset(3000, 1),
                )
            quantizationtools.TerminalQEvent(
                offset=durationtools.Offset(4000, 1),
                )

        Returns ``QEventSequence`` instance.
        '''
        from abjad.tools import quantizationtools
        if fuse_silences:
            durations = [
                x for x in
                sequencetools.Sequence(milliseconds).sum_by_sign(sign=[-1])
                if x
                ]
        else:
            durations = milliseconds
        offsets = mathtools.cumulative_sums([abs(x) for x in durations])
        q_events = []
        for pair in zip(offsets, durations):
            offset = durationtools.Offset(pair[0])
            duration = pair[1]
            if duration < 0: # negative duration indicates silence
                q_event = quantizationtools.SilentQEvent(offset)
            else:
                q_event = quantizationtools.PitchedQEvent(offset, [0])
            q_events.append(q_event)
        q_events.append(quantizationtools.TerminalQEvent(
            durationtools.Offset(offsets[-1])))
        return cls(q_events)

    @classmethod
    def from_millisecond_offsets(cls, offsets):
        r'''Convert millisecond offsets ``offsets`` into a ``QEventSequence``:

        ::

            >>> offsets = [0, 250, 750, 1750, 3000, 4000]

        ::

            >>> sequence = \
            ...     quantizationtools.QEventSequence.from_millisecond_offsets(
            ...     offsets)

        ::

            >>> for q_event in sequence:
            ...     print(format(q_event, 'storage'))
            ...
            quantizationtools.PitchedQEvent(
                offset=durationtools.Offset(0, 1),
                pitches=(
                    pitchtools.NamedPitch("c'"),
                    ),
                )
            quantizationtools.PitchedQEvent(
                offset=durationtools.Offset(250, 1),
                pitches=(
                    pitchtools.NamedPitch("c'"),
                    ),
                )
            quantizationtools.PitchedQEvent(
                offset=durationtools.Offset(750, 1),
                pitches=(
                    pitchtools.NamedPitch("c'"),
                    ),
                )
            quantizationtools.PitchedQEvent(
                offset=durationtools.Offset(1750, 1),
                pitches=(
                    pitchtools.NamedPitch("c'"),
                    ),
                )
            quantizationtools.PitchedQEvent(
                offset=durationtools.Offset(3000, 1),
                pitches=(
                    pitchtools.NamedPitch("c'"),
                    ),
                )
            quantizationtools.TerminalQEvent(
                offset=durationtools.Offset(4000, 1),
                )

        Returns ``QEventSequence`` instance.
        '''
        from abjad.tools import quantizationtools
        q_events = [quantizationtools.PitchedQEvent(x, [0])
            for x in offsets[:-1]]
        q_events.append(quantizationtools.TerminalQEvent(offsets[-1]))
        return cls(q_events)

    @classmethod
    def from_millisecond_pitch_pairs(cls, pairs):
        r'''Convert millisecond-duration:pitch pairs ``pairs`` into a
        ``QEventSequence``:

        ::

            >>> durations = [250, 500, 1000, 1250, 1000]
            >>> pitches = [(0,), None, (2, 3), None, (1,)]
            >>> pairs = tuple(zip(durations, pitches))

        ::

            >>> sequence = \
            ...     quantizationtools.QEventSequence.from_millisecond_pitch_pairs(
            ...     pairs)

        ::

            >>> for q_event in sequence:
            ...     print(format(q_event, 'storage'))
            ...
            quantizationtools.PitchedQEvent(
                offset=durationtools.Offset(0, 1),
                pitches=(
                    pitchtools.NamedPitch("c'"),
                    ),
                )
            quantizationtools.SilentQEvent(
                offset=durationtools.Offset(250, 1),
                )
            quantizationtools.PitchedQEvent(
                offset=durationtools.Offset(750, 1),
                pitches=(
                    pitchtools.NamedPitch("d'"),
                    pitchtools.NamedPitch("ef'"),
                    ),
                )
            quantizationtools.SilentQEvent(
                offset=durationtools.Offset(1750, 1),
                )
            quantizationtools.PitchedQEvent(
                offset=durationtools.Offset(3000, 1),
                pitches=(
                    pitchtools.NamedPitch("cs'"),
                    ),
                )
            quantizationtools.TerminalQEvent(
                offset=durationtools.Offset(4000, 1),
                )

        Returns ``QEventSequence`` instance.
        '''
        from abjad.tools import quantizationtools
        assert isinstance(pairs, collections.Iterable)
        assert all(isinstance(x, collections.Iterable) for x in pairs)
        assert all(len(x) == 2 for x in pairs)
        assert all(0 < x[0] for x in pairs)
        for pair in pairs:
            assert isinstance(pair[1], (
                numbers.Number, type(None), collections.Iterable))
            if isinstance(pair[1], collections.Iterable):
                assert 0 < len(pair[1])
                assert all(isinstance(x, numbers.Number) for x in pair[1])
        # fuse silences
        g = itertools.groupby(pairs, lambda x: x[1] is not None)
        groups = []
        for value, group in g:
            if value:
                groups.extend(list(group))
            else:
                duration = sum(x[0] for x in group)
                groups.append((duration, None))
        # find offsets
        offsets = mathtools.cumulative_sums([abs(x[0]) for x in groups])
        # build QEvents
        q_events = []
        for pair in zip(offsets, groups):
            offset = durationtools.Offset(pair[0])
            pitches = pair[1][1]
            if isinstance(pitches, collections.Iterable):
                assert all(isinstance(x, numbers.Number) for x in pitches)
                q_events.append(quantizationtools.PitchedQEvent(offset, pitches))
            elif isinstance(pitches, type(None)):
                q_events.append(quantizationtools.SilentQEvent(offset))
            elif isinstance(pitches, numbers.Number):
                q_events.append(quantizationtools.PitchedQEvent(offset, [pitches]))
        q_events.append(quantizationtools.TerminalQEvent(
            durationtools.Offset(offsets[-1])))
        return cls(q_events)

    @classmethod
    def from_tempo_scaled_durations(cls, durations, tempo=None):
        r'''Convert ``durations``, scaled by ``tempo``
        into a ``QEventSequence``:

        ::

            >>> tempo = Tempo((1, 4), 174)
            >>> durations = [(1, 4), (-3, 16), (1, 16), (-1, 2)]

        ::

            >>> sequence = \
            ...     quantizationtools.QEventSequence.from_tempo_scaled_durations(
            ...     durations, tempo=tempo)

        ::

            >>> for q_event in sequence:
            ...     print(format(q_event, 'storage'))
            ...
            quantizationtools.PitchedQEvent(
                offset=durationtools.Offset(0, 1),
                pitches=(
                    pitchtools.NamedPitch("c'"),
                    ),
                )
            quantizationtools.SilentQEvent(
                offset=durationtools.Offset(10000, 29),
                )
            quantizationtools.PitchedQEvent(
                offset=durationtools.Offset(17500, 29),
                pitches=(
                    pitchtools.NamedPitch("c'"),
                    ),
                )
            quantizationtools.SilentQEvent(
                offset=durationtools.Offset(20000, 29),
                )
            quantizationtools.TerminalQEvent(
                offset=durationtools.Offset(40000, 29),
                )

        Returns ``QEventSequence`` instance.
        '''
        from abjad.tools import quantizationtools
        durations = [durationtools.Duration(x) for x in durations]
        assert isinstance(tempo, indicatortools.Tempo)
        durations = [
            x for x in
            sequencetools.Sequence(durations).sum_by_sign(sign=[-1])
            if x
            ]
        durations = [tempo.duration_to_milliseconds(_) for _ in durations]
        offsets = mathtools.cumulative_sums([abs(_) for _ in durations])
        q_events = []
        for pair in zip(offsets, durations):
            offset = durationtools.Offset(pair[0])
            duration = pair[1]
            if duration < 0: # negative duration indicates silence
                q_event = quantizationtools.SilentQEvent(offset)
            else: # otherwise, use middle-C
                q_event = quantizationtools.PitchedQEvent(offset, [0])
            q_events.append(q_event)
        # insert terminating silence QEvent
        q_events.append(quantizationtools.TerminalQEvent(offsets[-1]))
        return cls(q_events)

    @classmethod
    def from_tempo_scaled_leaves(cls, leaves, tempo=None):
        r'''Convert ``leaves``, optionally with ``tempo`` into a
        ``QEventSequence``:

        ::

            >>> staff = Staff("c'4 <d' fs'>8. r16 gqs'2")
            >>> tempo = Tempo((1, 4), 72)

        ::

            >>> sequence = \
            ...     quantizationtools.QEventSequence.from_tempo_scaled_leaves(
            ...     staff[:], tempo=tempo)

        ::

            >>> for q_event in sequence:
            ...     print(format(q_event, 'storage'))
            ...
            quantizationtools.PitchedQEvent(
                offset=durationtools.Offset(0, 1),
                pitches=(
                    pitchtools.NamedPitch("c'"),
                    ),
                )
            quantizationtools.PitchedQEvent(
                offset=durationtools.Offset(2500, 3),
                pitches=(
                    pitchtools.NamedPitch("d'"),
                    pitchtools.NamedPitch("fs'"),
                    ),
                )
            quantizationtools.SilentQEvent(
                offset=durationtools.Offset(4375, 3),
                )
            quantizationtools.PitchedQEvent(
                offset=durationtools.Offset(5000, 3),
                pitches=(
                    pitchtools.NamedPitch("gqs'"),
                    ),
                )
            quantizationtools.TerminalQEvent(
                offset=durationtools.Offset(10000, 3),
                )

        If ``tempo`` is ``None``, all leaves in ``leaves`` must
        have an effective, non-imprecise tempo.
        The millisecond-duration of each leaf will be determined
        by its effective tempo.

        Return ``QEventSequence`` instance.
        '''
        from abjad.tools import quantizationtools
        from abjad.tools import selectiontools
        Selection = selectiontools.Selection
        assert Selection._all_are_contiguous_components_in_same_logical_voice(
            leaves)
        assert len(leaves)
        if tempo is None:
            assert leaves[0]._get_effective(indicatortools.Tempo) is not None
        #else:
        #    #tempo = indicatortools.Tempo(tempo)
        elif isinstance(tempo, indicatortools.Tempo):
            tempo = copy.copy(tempo)
        elif isinstance(tempo, tuple):
            tempo = indicatortools.Tempo(*tempo)
        else:
            raise TypeError(tempo)
        # sort by silence and tied leaves
        groups = []
        for rvalue, rgroup in itertools.groupby(
            leaves,
            lambda x: isinstance(x, (scoretools.Rest, scoretools.Skip))):
            if rvalue:
                groups.append(list(rgroup))
            else:
                for tvalue, tgroup in itertools.groupby(
                    rgroup, lambda x: x._get_logical_tie()):
                    groups.append(list(tgroup))
        # calculate lists of pitches and durations
        durations = []
        pitches = []
        for group in groups:
            # get millisecond cumulative duration
            if tempo is not None:
                duration = sum(
                    tempo.duration_to_milliseconds(x._get_duration())
                    for x in group)
            else:
                duration = sum(x._get_effective(
                    indicatortools.Tempo).duration_to_milliseconds(
                    x._get_duration())
                    for x in group)
            durations.append(duration)
            # get pitch of first leaf in group
            if isinstance(group[0], (scoretools.Rest, scoretools.Skip)):
                pitch = None
            elif isinstance(group[0], scoretools.Note):
                pitch = group[0].written_pitch.pitch_number
            else: # chord
                pitch = [x.written_pitch.pitch_number
                    for x in group[0].note_heads]
            pitches.append(pitch)
        # convert durations and pitches to QEvents and return
        return cls.from_millisecond_pitch_pairs(
            tuple(zip(durations, pitches)))

    ### PUBLIC PROPERTIES ###

    @property
    def duration_in_ms(self):
        r'''Duration in milliseconds of the ``QEventSequence``:

        ::

            >>> durations = (1000, -500, 1250, -500, 750)

        ::

            >>> sequence = \
            ...     quantizationtools.QEventSequence.from_millisecond_durations(
            ...     durations)

        ::

            >>> sequence.duration_in_ms
            Duration(4000, 1)

        Return ``Duration`` instance.
        '''
        return durationtools.Duration(self[-1].offset)

    @property
    def sequence(self):
        r'''Sequence of q-events.

        ::

            >>> for q_event in sequence.sequence:
            ...     print(format(q_event, 'storage'))
            ...
            quantizationtools.PitchedQEvent(
                offset=durationtools.Offset(0, 1),
                pitches=(
                    pitchtools.NamedPitch("c'"),
                    ),
                )
            quantizationtools.SilentQEvent(
                offset=durationtools.Offset(1000, 1),
                )
            quantizationtools.PitchedQEvent(
                offset=durationtools.Offset(1500, 1),
                pitches=(
                    pitchtools.NamedPitch("c'"),
                    ),
                )
            quantizationtools.SilentQEvent(
                offset=durationtools.Offset(2750, 1),
                )
            quantizationtools.PitchedQEvent(
                offset=durationtools.Offset(3250, 1),
                pitches=(
                    pitchtools.NamedPitch("c'"),
                    ),
                )
            quantizationtools.TerminalQEvent(
                offset=durationtools.Offset(4000, 1),
                )

        Returns tuple.
        '''
        return self._sequence



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_pitchtools_NamedPitch_from_pitch_carrier_01():

    pitch = NamedPitch('df', 5)
    named_pitch = NamedPitch.from_pitch_carrier(pitch)

    assert named_pitch == NamedPitch('df', 5)


def test_pitchtools_NamedPitch_from_pitch_carrier_02():

    note = Note(('df', 5), (1, 4))
    named_pitch = NamedPitch.from_pitch_carrier(note)

    assert named_pitch == NamedPitch('df', 5)


def test_pitchtools_NamedPitch_from_pitch_carrier_03():

    note = Note(('df', 5), (1, 4))
    named_pitch = NamedPitch.from_pitch_carrier(note.note_head)

    assert named_pitch == NamedPitch('df', 5)


def test_pitchtools_NamedPitch_from_pitch_carrier_04():

    chord = Chord([('df', 5)], (1, 4))
    named_pitch = NamedPitch.from_pitch_carrier(chord)

    assert named_pitch == NamedPitch('df', 5)


def test_pitchtools_NamedPitch_from_pitch_carrier_05():

    note = Note(None, (1, 4))
    statement = 't = NamedPitch.from_pitch_carrier(note)'
    assert pytest.raises(Exception, statement)


def test_pitchtools_NamedPitch_from_pitch_carrier_06():

    chord = Chord([], (1, 4))
    statement = 't = NamedPitch.from_pitch_carrier(chord)'
    assert pytest.raises(Exception, statement)


def test_pitchtools_NamedPitch_from_pitch_carrier_07():

    chord = Chord([0, 2, 11], (1, 4))
    statement = 't = NamedPitch.from_pitch_carrier(chord)'
    assert pytest.raises(Exception, statement)


def test_pitchtools_NamedPitch_from_pitch_carrier_08():

    pitch = NamedPitch.from_pitch_carrier(13)

    assert pitch == NamedPitch(13)



# -*- coding: utf-8 -*-
from abjad.tools.abctools.AbjadObject import AbjadObject


class AbjadValueObject(AbjadObject):
    r'''Abstract base class for classes which compare equally based on their
    storage format.
    '''

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### SPECIAL METHODS ###

    def __copy__(self, *arguments):
        r'''Copies Abjad value object.

        Returns new Abjad value object.
        '''
        from abjad.tools.topleveltools import new
        return new(self)

    def __eq__(self, argument):
        r'''Is true when all initialization values of Abjad value object equal
        the initialization values of `argument`.

        Returns true or false.
        '''
        from abjad.tools import systemtools
        return systemtools.TestManager.compare_objects(self, argument)

    def __hash__(self):
        r'''Hashes Abjad value object.

        Returns integer.
        '''
        from abjad.tools import systemtools
        hash_values = systemtools.StorageFormatAgent(self).get_hash_values()
        try:
            result = hash(hash_values)
        except TypeError:
            message = 'unhashable type: {}'.format(self)
            raise TypeError(message)
        return result



# -*- coding: utf-8 -*-
from abjad import *


def test_selectiontools_VerticalMoment___repr___01():
    r'''Vertical moment repr returns a nonempty string.
    '''

    vertical_moment = inspect_(Note('c4')).get_vertical_moment()
    representation = repr(vertical_moment)

    assert isinstance(representation, str) and 0 < len(representation)



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_scoretools_FixedDurationTuplet_trim_01():

    tuplet = scoretools.FixedDurationTuplet(Duration(2, 4), "c'8 d'8 e'8")
    tuplet.trim(0)

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \tweak edge-height #'(0.7 . 0)
        \times 4/3 {
            d'8
            e'8
        }
        '''
        )

    assert inspect_(tuplet).is_well_formed()


def test_scoretools_FixedDurationTuplet_trim_02():

    tuplet = scoretools.FixedDurationTuplet(Duration(2, 4), "c'8 d'8 e'8")
    tuplet.trim(1)

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \tweak edge-height #'(0.7 . 0)
        \times 4/3 {
            c'8
            e'8
        }
        '''
        )

    assert inspect_(tuplet).is_well_formed()


def test_scoretools_FixedDurationTuplet_trim_03():

    tuplet = scoretools.FixedDurationTuplet(Duration(2, 4), "c'8 d'8 e'8")
    assert pytest.raises(Exception, 'tuplet.trim(99)')


def test_scoretools_FixedDurationTuplet_trim_04():

    tuplet = scoretools.FixedDurationTuplet(Duration(2, 4), "c'8 d'8 e'8")
    tuplet.trim(0, 0)

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \times 4/3 {
            c'8
            d'8
            e'8
        }
        '''
        )

    assert inspect_(tuplet).is_well_formed()


def test_scoretools_FixedDurationTuplet_trim_05():

    tuplet = scoretools.FixedDurationTuplet(Duration(2, 4), "c'8 d'8 e'8")
    tuplet.trim(0, 1)

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \tweak edge-height #'(0.7 . 0)
        \times 4/3 {
            d'8
            e'8
        }
        '''
        )

    assert inspect_(tuplet).is_well_formed()


def test_scoretools_FixedDurationTuplet_trim_06():

    tuplet = scoretools.FixedDurationTuplet(Duration(2, 4), "c'8 d'8 e'8")
    tuplet.trim(1, 2)

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak text #tuplet-number::calc-fraction-text
        \tweak edge-height #'(0.7 . 0)
        \times 4/3 {
            c'8
            e'8
        }
        '''
        )

    assert inspect_(tuplet).is_well_formed()


def test_scoretools_FixedDurationTuplet_trim_07():
    r'''Trimming all leaves raises an exception.
    '''

    tuplet = scoretools.FixedDurationTuplet(Duration(2, 4), "c'8 d'8 e'8")
    tuplet.trim(1, 2)

    pytest.raises(Exception, 'tuplet.trim(0, 99)')



# -*- coding: utf-8 -*-
from abjad import *
import pytest


def test_scoretools_get_measure_01():

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 || 2/8 g'8 a'8 |")

    assert scoretools.get_measure(staff, 1) is \
        staff[0]
    assert scoretools.get_measure(staff, 2) is \
        staff[1]
    assert scoretools.get_measure(staff, 3) is \
        staff[2]


def test_scoretools_get_measure_02():

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 || 2/8 g'8 a'8 |")

    assert pytest.raises(
        ValueError,
        'scoretools.get_measure(staff, -1)',
        )



# -*- coding: utf-8 -*-
import six
from abjad.tools import systemtools
from abjad.tools import datastructuretools


# load constants into __builtins__ namespace
if six.PY3:
    import builtins
else:
    import __builtin__ as builtins
builtins.Identity = datastructuretools.OrdinalConstant('value', 0, 'Identity')
builtins.Less = datastructuretools.OrdinalConstant('value', -1, 'Less')
builtins.More = datastructuretools.OrdinalConstant('value', 1, 'More')
builtins.Exact = datastructuretools.OrdinalConstant('value', 0, 'Exact')
builtins.Left = datastructuretools.OrdinalConstant('x', -1, 'Left')
builtins.Right = datastructuretools.OrdinalConstant('x', 1, 'Right')
builtins.Both = datastructuretools.OrdinalConstant('x', 0, 'Both')
builtins.Center = datastructuretools.OrdinalConstant('y', 0, 'Center')
builtins.Up = datastructuretools.OrdinalConstant('y', 1, 'Up')
builtins.Down = datastructuretools.OrdinalConstant('y', -1, 'Down')
builtins.Top = datastructuretools.OrdinalConstant('y', 1, 'Top')
builtins.Bottom = datastructuretools.OrdinalConstant('y', -1, 'Bottom')
from abjad.tools import mathtools
builtins.Infinity = mathtools.Infinity()
builtins.NegativeInfinity = mathtools.NegativeInfinity()
del(builtins)
del(six)

systemtools.ImportManager.import_structured_package(
    __path__[0],
    globals(),
    delete_systemtools=False,
    ignored_names=['abjadbooktools'],
    )



# -*- coding: utf-8 -*-
from abjad import *


def test_indicatortools_Annotation___init___01():
    r'''Initializes annotation with dictionary.
    '''

    annotation = indicatortools.Annotation('special dictionary', {})
    assert isinstance(annotation, indicatortools.Annotation)


def test_indicatortools_Annotation___init___02():
    r'''Initializes annotation with only one argument.
    '''

    annotation = indicatortools.Annotation('foo')
    assert isinstance(annotation, indicatortools.Annotation)


def test_indicatortools_Annotation___init___03():
    r'''Initializes annotation from other annotation.
    '''

    annotation_1 = indicatortools.Annotation('foo', 'bar')
    annotation_2 = indicatortools.Annotation(annotation_1)

    assert annotation_1 == annotation_2
    assert annotation_1 is not annotation_2



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NumberedInterval_numbered_interval_class_01():

    assert pitchtools.NumberedIntervalClass(
        pitchtools.NumberedInterval(2)).number  == 2
    assert pitchtools.NumberedIntervalClass(
        pitchtools.NumberedInterval(14)).number == 2
    assert pitchtools.NumberedIntervalClass(
        pitchtools.NumberedInterval(26)).number == 2
    assert pitchtools.NumberedIntervalClass(
        pitchtools.NumberedInterval(38)).number == 2


def test_pitchtools_NumberedInterval_numbered_interval_class_02():

    assert pitchtools.NumberedIntervalClass(
        pitchtools.NumberedInterval(-2)).number == -2
    assert pitchtools.NumberedIntervalClass(
        pitchtools.NumberedInterval(-14)).number == -2
    assert pitchtools.NumberedIntervalClass(
        pitchtools.NumberedInterval(-26)).number == -2
    assert pitchtools.NumberedIntervalClass(
        pitchtools.NumberedInterval(-38)).number == -2



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools.lilypondparsertools import LilyPondParser


def test_lilypondparsertools_LilyPondParser__contexts__PianoStaff_01():

    target = StaffGroup([
        Staff(scoretools.make_notes([0, 2, 4, 5, 7], (1, 8))),
        Staff(scoretools.make_notes([0, 2, 4, 5, 7], (1, 8)))
    ])
    target.context_name = 'PianoStaff'

    assert format(target) == stringtools.normalize(
        r'''
        \new PianoStaff <<
            \new Staff {
                c'8
                d'8
                e'8
                f'8
                g'8
            }
            \new Staff {
                c'8
                d'8
                e'8
                f'8
                g'8
            }
        >>
        '''
        )

    parser = LilyPondParser()
    result = parser(format(target))
    assert format(target) == format(result) and target is not result



# -*- coding: utf-8 -*-
from abjad.tools.pitchtools import NumberedInterval
from abjad.tools.pitchtools import IntervalSegment


def test_pitchtools_IntervalSegment_spread_01():
    mcis = IntervalSegment([1, 2, -3, 1, -2, 1])
    assert mcis.spread == NumberedInterval(4)


def test_pitchtools_IntervalSegment_spread_02():
    mcis = IntervalSegment([1, 1, 1, 2, -3, -2])
    assert mcis.spread == NumberedInterval(5)


def test_pitchtools_IntervalSegment_spread_03():
    mcis = IntervalSegment([1, 1, -2, 2, -3, 1])
    assert mcis.spread == NumberedInterval(3)



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools.quantizationtools import *


def test_quantizationtools_QGrid_subdivide_leaf_01():

    q_grid = QGrid()

    a = QEventProxy(PitchedQEvent(0, [0]), 0)
    b = QEventProxy(PitchedQEvent((9, 20), [1]), (9, 20))
    c = QEventProxy(PitchedQEvent((1, 2), [2]), (1, 2))
    d = QEventProxy(PitchedQEvent((11, 20), [3]), (11, 20))
    e = QEventProxy(PitchedQEvent(1, [4]), 1)

    q_grid.leaves[0].q_event_proxies.extend([a, b, c, d])
    q_grid.leaves[1].q_event_proxies.append(e)

    result = q_grid.subdivide_leaf(q_grid.leaves[0], (2, 3))

    assert result == [a, b, c, d]
    root_node = QGridContainer(
        children=[
            QGridLeaf(preprolated_duration=2, q_event_proxies=[]),
            QGridLeaf(preprolated_duration=3, q_event_proxies=[]),
            ],
        preprolated_duration=1
        )
    assert format(q_grid.root_node) == format(root_node)



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_Registration___init___01():
    r'''Initialize from items.
    '''

    mapping = pitchtools.Registration([('[A0, C4)', 15), ('[C4, C8)', 27)])
    assert isinstance(mapping, pitchtools.Registration)


def test_pitchtools_Registration___init___02():
    r'''Initialize from instance.
    '''

    mapping_1 = pitchtools.Registration([('[A0, C4)', 15), ('[C4, C8)', 27)])
    mapping_2 = pitchtools.Registration(mapping_1)

    assert isinstance(mapping_1, pitchtools.Registration)
    assert isinstance(mapping_2, pitchtools.Registration)
    assert mapping_1 == mapping_2


def test_pitchtools_Registration___init___03():
    r'''Initialize from named instance.
    '''

    mapping_1 = pitchtools.Registration(
        [('[A0, C4)', 15), ('[C4, C8)', 27)],
        )
    mapping_2 = pitchtools.Registration(mapping_1)

    assert isinstance(mapping_1, pitchtools.Registration)
    assert isinstance(mapping_2, pitchtools.Registration)
    assert mapping_1 == mapping_2


def test_pitchtools_Registration___init___04():
    r'''Initializeempty.
    '''

    mapping = pitchtools.Registration()
    assert isinstance(mapping, pitchtools.Registration)

    mapping = pitchtools.Registration([])
    assert isinstance(mapping, pitchtools.Registration)



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_make_repeated_notes_from_time_signature_01():
    r'''Make repeated notes from integer pair.
    '''

    notes = scoretools.make_repeated_notes_from_time_signature((5, 32), pitch="d''")
    staff = Staff(notes)

    assert inspect_(staff).is_well_formed()
    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            d''32
            d''32
            d''32
            d''32
            d''32
        }
        '''
        )


def test_scoretools_make_repeated_notes_from_time_signature_02():
    r'''Make repeated notes from time signature.
    '''

    time_signature = TimeSignature((5, 32))
    notes = scoretools.make_repeated_notes_from_time_signature(time_signature, pitch="d''")
    staff = Staff(notes)

    assert inspect_(staff).is_well_formed()
    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            d''32
            d''32
            d''32
            d''32
            d''32
        }
        '''
        )



# -*- coding: utf-8 -*-


def is_integer_pair(argument):
    r'''Is true when `argument` is an integer tuple of length 2. Otherwise
    false.

    ..  container:: example

        ::

            >>> mathtools.is_integer_pair((19, 20))
            True

        ::

            >>> mathtools.is_integer_pair(('some', 'text'))
            False

    Returns true or false.
    '''
    return (
        isinstance(argument, tuple) and
        len(argument) == 2 and
        all(isinstance(_, int) for _ in argument)
        )



# -*- coding: utf-8 -*-
from abjad import *


def test_tonalanalysistools_TonalAnalysisAgent_analyze_incomplete_chords_01():

    chord = Chord('<g b>4')
    chord_class = tonalanalysistools.RootedChordClass('g', 'major', 'triad', 'root')
    selection = tonalanalysistools.select(chord)
    assert selection.analyze_incomplete_chords() == [chord_class]

    chord = Chord('<g bf>4')
    chord_class = tonalanalysistools.RootedChordClass('g', 'minor', 'triad', 'root')
    selection = tonalanalysistools.select(chord)
    assert selection.analyze_incomplete_chords() == [chord_class]


def test_tonalanalysistools_TonalAnalysisAgent_analyze_incomplete_chords_02():

    chord = Chord('<f g b>4')
    chord_class = tonalanalysistools.RootedChordClass('g', 'dominant', 'seventh', 2)
    selection = tonalanalysistools.select(chord)
    assert selection.analyze_incomplete_chords() == [chord_class]

    chord = Chord('<fs g b>4')
    chord_class = tonalanalysistools.RootedChordClass('g', 'major', 'seventh', 2)
    selection = tonalanalysistools.select(chord)
    assert selection.analyze_incomplete_chords() == [chord_class]



# -*- coding: utf-8 -*-


def to_tridirectional_ordinal_constant(argument):
    r'''Changes `argument` to tridirectional ordinal constant.

    ..  container:: example

            >>> stringtools.to_tridirectional_ordinal_constant('^')
            Up

    ..  container:: example

        ::

            >>> stringtools.to_tridirectional_ordinal_constant('_')
            Down

    ..  container:: example

        ::

            >>> stringtools.to_tridirectional_ordinal_constant(1)
            Up

    ..  container:: example

        ::

            >>> stringtools.to_tridirectional_ordinal_constant(-1)
            Down

    Returns `argument` when `argument` is `Up`', `Center` or `Down`.

    Returns ordinal constant or none.
    '''
    lookup = {
        Up: Up,
        '^': Up,
        'up': Up,
        1: Up,
        Down: Down,
        '_': Down,
        'down': Down,
        -1: Down,
        Center: Center,
        '-': Center,
        0: Center,
        'center': Center,
        'default': Center,
        'neutral': Center,
        }
    if argument is None:
        return None
    elif argument in lookup:
        return lookup[argument]
    message = 'unrecognized expression: {!r}.'
    message = message.format(argument)
    raise ValueError(message)



# -*- coding: utf-8 -*-
from abjad.tools import pitchtools
from abjad.tools.spannertools.Spanner import Spanner
from abjad.tools.topleveltools import inspect_


class ComplexTrillSpanner(Spanner):
    r'''Complex trill spanner.

    ..  container:: example

        ::

            >>> staff = Staff("c'4 ~ c'8 d'8 r8 e'8 ~ e'8 r8")
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                c'4 ~
                c'8
                d'8
                r8
                e'8 ~
                e'8
                r8
            }

        ::

            >>> complex_trill = spannertools.ComplexTrillSpanner(
            ...     interval='P4',
            ...     )
            >>> attach(complex_trill, staff[:])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                \pitchedTrill
                c'4 ~ \startTrillSpan f'
                c'8
                <> \stopTrillSpan
                \once \override TrillSpanner.bound-details.left.text = \markup {
                    \null
                    }
                \pitchedTrill
                d'8 \startTrillSpan g'
                <> \stopTrillSpan
                r8
                \pitchedTrill
                e'8 ~ \startTrillSpan a'
                e'8
                <> \stopTrillSpan
                r8
            }

    Allows for specifying a trill pitch via a named interval.

    Avoids silences.

    Restarts the trill on every new pitched logical tie.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_interval',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        interval=None,
        overrides=None,
        ):
        Spanner.__init__(
            self,
            overrides=overrides,
            )
        if interval is not None:
            interval = pitchtools.NamedInterval(interval)
        self._interval = interval

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        new._interval = self.interval

    def _get_lilypond_format_bundle(self, leaf):
        from abjad.tools import lilypondnametools
        from abjad.tools import markuptools
        from abjad.tools import scoretools
        lilypond_format_bundle = self._get_basic_lilypond_format_bundle(leaf)
        prototype = (
            scoretools.Rest,
            scoretools.MultimeasureRest,
            scoretools.Skip,
            )
        if isinstance(leaf, prototype):
            return lilypond_format_bundle
        logical_tie = inspect_(leaf).get_logical_tie()
        if leaf is logical_tie.head:
            previous_leaf = leaf._get_leaf(-1)
            if (previous_leaf is not None and
                not isinstance(previous_leaf, prototype) and
                inspect_(previous_leaf).get_spanners(type(self))):
                grob_override = lilypondnametools.LilyPondGrobOverride(
                    grob_name='TrillSpanner',
                    is_once=True,
                    property_path=(
                        'bound-details',
                        'left',
                        'text',
                        ),
                    value=markuptools.Markup(r'\null'),
                    )
                string = grob_override.override_string
                lilypond_format_bundle.grob_overrides.append(string)
            if self.interval is not None:
                string = r'\pitchedTrill'
                lilypond_format_bundle.opening.spanners.append(string)
                if hasattr(leaf, 'written_pitch'):
                    written_pitch = leaf.written_pitch
                elif hasattr(leaf, 'written_pitches'):
                    if 0 < self.interval.semitones:
                        written_pitch = max(leaf.written_pitches)
                    elif self.interval.semitones < 0:
                        written_pitch = min(leaf.written_pitches)
                trill_pitch = written_pitch.transpose(self.interval)
                string = r'\startTrillSpan {!s}'.format(trill_pitch)
            else:
                string = r'\startTrillSpan'
            lilypond_format_bundle.right.trill_pitches.append(string)
        if leaf is logical_tie.tail:
            next_leaf = leaf._get_leaf(1)
            if next_leaf is not None:
                string = r'<> \stopTrillSpan'
                lilypond_format_bundle.after.commands.append(string)
            else:
                string = r'\stopTrillSpan'
                lilypond_format_bundle.right.spanner_stops.append(string)
        return lilypond_format_bundle

    ### PUBLIC PROPERTIES ###

    @property
    def interval(self):
        r'''Gets optional interval of trill spanner.

        ..  container:: example

            ::

                >>> staff = Staff("c'4 d'4 e'4 f'4")
                >>> interval = pitchtools.NamedInterval('m3')
                >>> complex_trill = spannertools.ComplexTrillSpanner(
                ...     interval=interval)
                >>> attach(complex_trill, staff[1:-1])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    c'4
                    \pitchedTrill
                    d'4 \startTrillSpan f'
                    <> \stopTrillSpan
                    \once \override TrillSpanner.bound-details.left.text = \markup {
                        \null
                        }
                    \pitchedTrill
                    e'4 \startTrillSpan g'
                    <> \stopTrillSpan
                    f'4
                }

            ::

                >>> complex_trill.interval
                NamedInterval('+m3')

        '''
        return self._interval



# -*- coding: utf-8 -*-
from abjad import *


def test_stringtools_is_snake_case_01():

    assert stringtools.is_snake_case('foo_bar_blah')
    assert stringtools.is_snake_case('foo_bar')
    assert stringtools.is_snake_case('foo')
    assert stringtools.is_snake_case('f')


def test_stringtools_is_snake_case_02():

    assert stringtools.is_snake_case('')


def test_stringtools_is_snake_case_03():

    assert not stringtools.is_snake_case('foo_Bar')
    assert not stringtools.is_snake_case('foo bar')
    assert not stringtools.is_snake_case('Foo_bar')
    assert not stringtools.is_snake_case('Foo_Bar')



# -*- coding: utf-8 -*-
from abjad import *


def test_indicatortools_Tempo_quarters_per_minute_01():
    r'''Maelzel metronome marking with integer.
    '''

    tempomark = Tempo(Duration(3, 32), 52)
    assert tempomark.quarters_per_minute == Duration(416, 3)


def test_indicatortools_Tempo_quarters_per_minute_02():
    r'''Maelzel metronome marking with float.
    '''

    tempomark = Tempo(Duration(3, 32), 52.5)
    assert tempomark.quarters_per_minute == 140.0



# -*- coding: utf-8 -*-
import collections
from abjad.tools import indicatortools
from abjad.tools import scoretools
from abjad.tools.topleveltools import attach
from abjad.tools.abctools.AbjadObject import AbjadObject


class GroupedRhythmicStavesScoreTemplate(AbjadObject):
    r'''Grouped rhythmic staves score template.

    ::

            >>> from abjad.tools.templatetools import *
            >>> template_class = GroupedRhythmicStavesScoreTemplate

    ..  container:: example

        **Example 1.** One voice per staff:

        ::

            >>> template_1 = template_class(staff_count=4)
            >>> score = template_1()
            >>> print(format(score))
            \context Score = "Grouped Rhythmic Staves Score" <<
                \context StaffGroup = "Grouped Rhythmic Staves Staff Group" <<
                    \context RhythmicStaff = "Staff 1" {
                        \context Voice = "Voice 1" {
                        }
                    }
                    \context RhythmicStaff = "Staff 2" {
                        \context Voice = "Voice 2" {
                        }
                    }
                    \context RhythmicStaff = "Staff 3" {
                        \context Voice = "Voice 3" {
                        }
                    }
                    \context RhythmicStaff = "Staff 4" {
                        \context Voice = "Voice 4" {
                        }
                    }
                >>
            >>

    ..  container:: example

        **Example 2.** More than one voice per staff:

        ::

            >>> template_2 = template_class(staff_count=[2, 1, 2])
            >>> score = template_2()
            >>> print(format(score))
            \context Score = "Grouped Rhythmic Staves Score" <<
                \context StaffGroup = "Grouped Rhythmic Staves Staff Group" <<
                    \context RhythmicStaff = "Staff 1" <<
                        \context Voice = "Voice 1-1" {
                        }
                        \context Voice = "Voice 1-2" {
                        }
                    >>
                    \context RhythmicStaff = "Staff 2" {
                        \context Voice = "Voice 2" {
                        }
                    }
                    \context RhythmicStaff = "Staff 3" <<
                        \context Voice = "Voice 3-1" {
                        }
                        \context Voice = "Voice 3-2" {
                        }
                    >>
                >>
            >>

    ..  container:: example

        **Example 3.** With percussion clefs attached.

            >>> template_3 = template_class(
            ...     staff_count=[2, 1, 2],
            ...     with_clefs=True,
            ...     )
            >>> score = template_3()
            >>> print(format(score))
            \context Score = "Grouped Rhythmic Staves Score" <<
                \context StaffGroup = "Grouped Rhythmic Staves Staff Group" <<
                    \context RhythmicStaff = "Staff 1" <<
                        \clef "percussion"
                        \context Voice = "Voice 1-1" {
                        }
                        \context Voice = "Voice 1-2" {
                        }
                    >>
                    \context RhythmicStaff = "Staff 2" {
                        \clef "percussion"
                        \context Voice = "Voice 2" {
                        }
                    }
                    \context RhythmicStaff = "Staff 3" <<
                        \clef "percussion"
                        \context Voice = "Voice 3-1" {
                        }
                        \context Voice = "Voice 3-2" {
                        }
                    >>
                >>
            >>

    '''

    ### INITIALIZER ###

    def __init__(self, staff_count=2, with_clefs=None):
        assert isinstance(staff_count, (int, list))
        self.context_name_abbreviations = collections.OrderedDict()
        self._staff_count = staff_count
        if with_clefs is not None:
            with_clefs = bool(with_clefs)
        self._with_clefs = with_clefs

    ### SPECIAL METHODS ###

    def __call__(self):
        r'''Calls score template.

        ..  container:: example

            **Example 1.** Call first template:

            ::

                >>> score_1 = template_1()

            ::

                >>> print(format(score_1))
                \context Score = "Grouped Rhythmic Staves Score" <<
                    \context StaffGroup = "Grouped Rhythmic Staves Staff Group" <<
                        \context RhythmicStaff = "Staff 1" {
                            \context Voice = "Voice 1" {
                            }
                        }
                        \context RhythmicStaff = "Staff 2" {
                            \context Voice = "Voice 2" {
                            }
                        }
                        \context RhythmicStaff = "Staff 3" {
                            \context Voice = "Voice 3" {
                            }
                        }
                        \context RhythmicStaff = "Staff 4" {
                            \context Voice = "Voice 4" {
                            }
                        }
                    >>
                >>

        ..  container:: example

            **Example 2.** Call second template:

            ::

                >>> score_2 = template_2()

            ::

                >>> print(format(score_2))
                \context Score = "Grouped Rhythmic Staves Score" <<
                    \context StaffGroup = "Grouped Rhythmic Staves Staff Group" <<
                        \context RhythmicStaff = "Staff 1" <<
                            \context Voice = "Voice 1-1" {
                            }
                            \context Voice = "Voice 1-2" {
                            }
                        >>
                        \context RhythmicStaff = "Staff 2" {
                            \context Voice = "Voice 2" {
                            }
                        }
                        \context RhythmicStaff = "Staff 3" <<
                            \context Voice = "Voice 3-1" {
                            }
                            \context Voice = "Voice 3-2" {
                            }
                        >>
                    >>
                >>

        Returns score.
        '''
        staves = []
        if isinstance(self.staff_count, int):
            for index in range(self.staff_count):
                number = index + 1
                name = 'Voice {}'.format(number)
                voice = scoretools.Voice(name=name)
                name='Staff {}'.format(number)
                staff = scoretools.Staff([voice], name=name)
                staff.context_name = 'RhythmicStaff'
                staves.append(staff)
                key = 'v{}'.format(number)
                self.context_name_abbreviations[key] = voice.name
        elif isinstance(self.staff_count, list):
            for staff_index, voice_count in enumerate(self.staff_count):
                staff_number = staff_index + 1
                name = 'Staff {}'.format(staff_number)
                staff = scoretools.Staff(name=name)
                staff.context_name = 'RhythmicStaff'
                assert 1 <= voice_count
                for voice_index in range(voice_count):
                    voice_number = voice_index + 1
                    if voice_count == 1:
                        voice_identifier = str(staff_number)
                    else:
                        voice_identifier = '{}-{}'.format(
                            staff_number, voice_number)
                        staff.is_simultaneous = True
                    name = 'Voice {}'.format(voice_identifier)
                    voice = scoretools.Voice(name=name)
                    staff.append(voice)
                    key = 'v{}'.format(voice_identifier)
                    self.context_name_abbreviations[key] = voice.name
                staves.append(staff)
        if self.with_clefs:
            for staff in staves:
                attach(indicatortools.Clef('percussion'), staff)
        grouped_rhythmic_staves_staff_group = scoretools.StaffGroup(
            staves,
            name='Grouped Rhythmic Staves Staff Group',
            )
        grouped_rhythmic_staves_score = scoretools.Score(
            [grouped_rhythmic_staves_staff_group],
            name='Grouped Rhythmic Staves Score',
            )
        return grouped_rhythmic_staves_score

    ### PUBLIC PROPERTIES ###

    @property
    def staff_count(self):
        r'''Score template staff count.

        ::

            >>> template_1.staff_count
            4

        Returns nonnegative integer.
        '''
        return self._staff_count

    @property
    def with_clefs(self):
        r'''Is true if template should attach percussion clefs. Otherwise
        false.

        Returns true or false.
        '''
        return self._with_clefs



# -*- coding: utf-8 -*-
from abjad import *


def test_indicatortools_TimeSignature_with_power_of_two_denominator_01():
    r'''Make n/12 time_signatures into n/8 time_signatures, where possible.
    '''

    assert TimeSignature((1, 12)).with_power_of_two_denominator() == (1, 12)
    assert TimeSignature((2, 12)).with_power_of_two_denominator() == (2, 12)
    assert TimeSignature((3, 12)).with_power_of_two_denominator() == (2, 8)
    assert TimeSignature((4, 12)).with_power_of_two_denominator() == (4, 12)
    assert TimeSignature((5, 12)).with_power_of_two_denominator() == (5, 12)
    assert TimeSignature((6, 12)).with_power_of_two_denominator() == (4, 8)


def test_indicatortools_TimeSignature_with_power_of_two_denominator_02():
    r'''Make n/14 time_signatures into n/8 time_signatures, where possible.
    '''

    assert TimeSignature((1, 14)).with_power_of_two_denominator() == (1, 14)
    assert TimeSignature((2, 14)).with_power_of_two_denominator() == (2, 14)
    assert TimeSignature((3, 14)).with_power_of_two_denominator() == (3, 14)
    assert TimeSignature((4, 14)).with_power_of_two_denominator() == (4, 14)
    assert TimeSignature((5, 14)).with_power_of_two_denominator() == (5, 14)
    assert TimeSignature((6, 14)).with_power_of_two_denominator() == (6, 14)
    assert TimeSignature((7, 14)).with_power_of_two_denominator() == (4, 8)


def test_indicatortools_TimeSignature_with_power_of_two_denominator_03():
    r'''Make n/24 time_signatures into n/16 time_signatures, where possible.
    '''

    assert TimeSignature((1, 24)).with_power_of_two_denominator() == (1, 24)
    assert TimeSignature((2, 24)).with_power_of_two_denominator() == (2, 24)
    assert TimeSignature((3, 24)).with_power_of_two_denominator() == (2, 16)
    assert TimeSignature((4, 24)).with_power_of_two_denominator() == (4, 24)
    assert TimeSignature((5, 24)).with_power_of_two_denominator() == (5, 24)
    assert TimeSignature((6, 24)).with_power_of_two_denominator() == (4, 16)
    assert TimeSignature((7, 24)).with_power_of_two_denominator() == (7, 24)
    assert TimeSignature((8, 24)).with_power_of_two_denominator() == (8, 24)


def test_indicatortools_TimeSignature_with_power_of_two_denominator_04():
    r'''Make n/24 time_signatures into n/8 time_signatures, where possible.
    '''

    assert TimeSignature((1, 24)).with_power_of_two_denominator(Multiplier(99)) == (1, 24)
    assert TimeSignature((2, 24)).with_power_of_two_denominator(Multiplier(99)) == (2, 24)
    assert TimeSignature((3, 24)).with_power_of_two_denominator(Multiplier(99)) == (1, 8)
    assert TimeSignature((4, 24)).with_power_of_two_denominator(Multiplier(99)) == (4, 24)
    assert TimeSignature((5, 24)).with_power_of_two_denominator(Multiplier(99)) == (5, 24)
    assert TimeSignature((6, 24)).with_power_of_two_denominator(Multiplier(99)) == (2, 8)
    assert TimeSignature((7, 24)).with_power_of_two_denominator(Multiplier(99)) == (7, 24)
    assert TimeSignature((8, 24)).with_power_of_two_denominator(Multiplier(99)) == (8, 24)



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NamedInterval___repr___01():

    interval = pitchtools.NamedInterval('perfect', 1)
    repr = interval.__repr__()
    assert  repr == "NamedInterval('P1')"

    interval = pitchtools.NamedInterval('augmented', 1)
    repr = interval.__repr__()
    assert  repr == "NamedInterval('+aug1')"

    interval = pitchtools.NamedInterval('minor', 2)
    repr = interval.__repr__()
    assert  repr == "NamedInterval('+m2')"

    interval = pitchtools.NamedInterval('major', 2)
    repr = interval.__repr__()
    assert  repr == "NamedInterval('+M2')"

    interval = pitchtools.NamedInterval('minor', 3)
    repr = interval.__repr__()
    assert  repr == "NamedInterval('+m3')"


def test_pitchtools_NamedInterval___repr___02():

    interval = pitchtools.NamedInterval('perfect', -1)
    repr = interval.__repr__()
    assert  repr == "NamedInterval('P1')"

    interval = pitchtools.NamedInterval('augmented', -1)
    repr = interval.__repr__()
    assert  repr == "NamedInterval('-aug1')"

    interval = pitchtools.NamedInterval('minor', -2)
    repr = interval.__repr__()
    assert  repr == "NamedInterval('-m2')"

    interval = pitchtools.NamedInterval('major', -2)
    repr = interval.__repr__()
    assert  repr == "NamedInterval('-M2')"

    interval = pitchtools.NamedInterval('minor', -3)
    repr = interval.__repr__()
    assert  repr == "NamedInterval('-m3')"



# -*- coding: utf-8 -*-
import enum
from abjad.tools import stringtools
from abjad.tools import systemtools


class Enumeration(enum.IntEnum):
    r'''An enumeration.

    ::

        >>> class Colors(datastructuretools.Enumeration):
        ...     RED = 1
        ...     BLUE = 2
        ...     LIGHT_GREEN = 3
        ...

    ::

        >>> color = Colors.RED
        >>> print(repr(color))
        Colors.RED

    ::

        >>> Colors.from_expr('light green')
        Colors.LIGHT_GREEN

    '''

    ### SPECIAL METHODS ###

    def __dir__(self):
        names = [
            '__class__',
            '__doc__',
            '__format__',
            '__members__',
            '__module__',
            '__repr__',
            '_get_format_specification',
            'from_expr',
            ]
        names += self._member_names_
        names += [
            ]
        return sorted(names)

    def __format__(self, format_specification=''):
        r'''Formats enumeration.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatAgent(self).get_storage_format()
        return str(self)

    def __repr__(self):
        r'''Gets interpreter representation of enumeration.

        Returns string.
        '''
        from abjad.tools import systemtools
        return systemtools.StorageFormatAgent(self).get_repr_format()

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        agent = systemtools.StorageFormatAgent(self)
        repr_text = '{}.{}'.format(type(self).__name__, self.name)
        storage_format_text = '{}.{}'.format(
            agent.get_tools_package_name(),
            repr_text,
            )
        return systemtools.FormatSpecification(
            client=self,
            repr_text=repr_text,
            storage_format_text=storage_format_text,
            )

    ### PUBLIC METHODS ###

    @classmethod
    def from_expr(cls, argument):
        r'''Convenience constructor for enumerations.

        Returns new enumeration item.
        '''
        if isinstance(argument, cls):
            return argument
        elif isinstance(argument, int):
            return cls(argument)
        elif isinstance(argument, str):
            argument = argument.strip()
            argument = stringtools.to_snake_case(argument)
            argument = argument.upper()
            try:
                return cls[argument]
            except KeyError:
                return cls[argument.replace('_', '')]
        elif argument is None:
            return cls(0)
        message = 'Cannot instantiate {} from {}.'.format(cls.__name__, argument)
        raise ValueError(message)



# -*- coding: utf-8 -*-
import os
import shutil
import subprocess
import sys
import tempfile


class Play(object):
    r'''IPython replacement callable for `topleveltools.show()`.

    Integrates audio rendering of Abjad MIDI files into IPython notebooks
    using `timidity`.
    '''

    ### SPECIAL METHODS ###

    def __call__(self, argument):
        '''Render `argument` as audio and display it in the IPython notebook
        as an <audio> tag.
        '''
        from abjad.tools import systemtools
        from abjad.tools import topleveltools
        assert hasattr(argument, '__illustrate__')
        assert systemtools.IOManager.find_executable('lilypond')
        assert systemtools.IOManager.find_executable('timidity')
        has_vorbis = self._check_for_vorbis()
        temp_directory = tempfile.mkdtemp()
        midi_file_path = os.path.join(temp_directory, 'out.midi')
        result = topleveltools.persist(argument).as_midi(midi_file_path)
        midi_file_path, format_time, render_time = result
        if has_vorbis:
            audio_file_path = os.path.join(temp_directory, 'out.ogg')
        else:
            audio_file_path = os.path.join(temp_directory, 'out.aif')
        encoded_audio = self._get_audio_as_base64(
            midi_file_path,
            audio_file_path,
            has_vorbis,
            )
        if encoded_audio is not None:
            self._display_audio_tag(encoded_audio, has_vorbis)
        shutil.rmtree(temp_directory)

    ### PRIVATE METHODS ###

    def _check_for_vorbis(self):
        has_vorbis = False
        output = subprocess.check_output('timidity --help', shell=True)
        if sys.version_info[0] == 3:
            output = output.decode('utf-8')
        for line in output.splitlines():
            for part in line.split():
                if part == '-Ov':
                    has_vorbis = True
                    break
        return has_vorbis

    def _display_audio_tag(self, encoded_audio, has_vorbis):
        from IPython.core.display import display_html
        if has_vorbis:
            mime_type = 'audio/ogg'
        else:
            mime_type = 'audio/aiff'
        audio_tag = '<audio controls type="{}" '
        audio_tag += 'src="data:{};base64,{}">'
        audio_tag = audio_tag.format(mime_type, mime_type, encoded_audio)
        display_html(audio_tag, raw=True)

    def _get_audio_as_base64(
        self,
        midi_file_path,
        audio_file_path,
        has_vorbis,
        ):
        if has_vorbis:
            output_flag = '-Ov'
        else:
            output_flag = '-Oa'
        command = 'timidity {midi_file_path} {output_flag} -o {audio_file_path}'.format(
            midi_file_path=midi_file_path,
            output_flag=output_flag,
            audio_file_path=audio_file_path,
            )
        result = subprocess.call(command, shell=True)
        if result == 0:
            encoded_audio = self._get_base64_from_file(audio_file_path)
            return encoded_audio
        message = 'timidity failed to render MIDI, result: {}'
        message = message.format(result)
        print(message)
        return None

    def _get_base64_from_file(self, file_name):
        '''Read the base64 representation of a file and encode for HTML.
        '''
        import base64
        with open(file_name, 'rb') as file_pointer:
            data = file_pointer.read()
            return base64.b64encode(data).decode('utf-8')



# -*- coding: utf-8 -*-
from abjad.tools import indicatortools
from abjad.tools import schemetools
from abjad.tools.topleveltools import override
from abjad.tools.topleveltools import set_


def make_floating_time_signature_lilypond_file(music=None):
    r'''Makes floating time signature LilyPond file.

    ..  container:: example

        ::

            >>> score = Score()
            >>> time_signature_context = scoretools.Context(
            ...     context_name='TimeSignatureContext',
            ...     )
            >>> durations = [(2, 8), (3, 8), (4, 8)]
            >>> measures = scoretools.make_spacer_skip_measures(durations)
            >>> time_signature_context.extend(measures)
            >>> score.append(time_signature_context)
            >>> staff = Staff()
            >>> staff.append(Measure((2, 8), "c'8 ( d'8 )"))
            >>> staff.append(Measure((3, 8), "e'8 ( f'8  g'8 )"))
            >>> staff.append(Measure((4, 8), "fs'4 ( e'8 d'8 )"))
            >>> score.append(staff)
            >>> lilypond_file = \
            ...     lilypondfiletools.make_floating_time_signature_lilypond_file(
            ...     score
            ...     )

        ::

            >>> print(format(lilypond_file)) # doctest: +SKIP
            % 2014-01-07 18:22

            \version "2.19.0"
            \language "english"

            #(set-default-paper-size "letter" 'portrait)
            #(set-global-staff-size 12)

            \header {}

            \layout {
                \accidentalStyle forget
                indent = #0
                ragged-right = ##t
                \context {
                    \name TimeSignatureContext
                    \type Engraver_group
                    \consists Axis_group_engraver
                    \consists Time_signature_engraver
                    \override TimeSignature.X-extent = #'(0 . 0)
                    \override TimeSignature.X-offset = #ly:self-alignment-interface::x-aligned-on-self
                    \override TimeSignature.Y-extent = #'(0 . 0)
                    \override TimeSignature.break-align-symbol = ##f
                    \override TimeSignature.break-visibility = #end-of-line-invisible
                    \override TimeSignature.font-size = #1
                    \override TimeSignature.self-alignment-X = #center
                    \override VerticalAxisGroup.default-staff-staff-spacing = #'((basic-distance . 0) (minimum-distance . 12) (padding . 6) (stretchability . 0))
                }
                \context {
                    \Score
                    \remove Bar_number_engraver
                    \accepts TimeSignatureContext
                    \override Beam.breakable = ##t
                    \override SpacingSpanner.strict-grace-spacing = ##t
                    \override SpacingSpanner.strict-note-spacing = ##t
                    \override SpacingSpanner.uniform-stretching = ##t
                    \override TupletBracket.bracket-visibility = ##t
                    \override TupletBracket.minimum-length = #3
                    \override TupletBracket.padding = #2
                    \override TupletBracket.springs-and-rods = #ly:spanner::set-spacing-rods
                    \override TupletNumber.text = #tuplet-number::calc-fraction-text
                    autoBeaming = ##f
                    proportionalNotationDuration = #(ly:make-moment 1 32)
                    tupletFullLength = ##t
                }
                \context {
                    \StaffGroup
                }
                \context {
                    \Staff
                    \remove Time_signature_engraver
                }
                \context {
                    \RhythmicStaff
                    \remove Time_signature_engraver
                }
            }

            \paper {
                left-margin = #20
                system-system-spacing = #'((basic-distance . 0) (minimum-distance . 0) (padding . 12) (stretchability . 0))
            }

            \score {
                \new Score <<
                    \new TimeSignatureContext {
                        {
                            \time 2/8
                            s1 * 1/4
                        }
                        {
                            \time 3/8
                            s1 * 3/8
                        }
                        {
                            \time 4/8
                            s1 * 1/2
                        }
                    }
                    \new Staff {
                        {
                            \time 2/8
                            c'8 (
                            d'8 )
                        }
                        {
                            \time 3/8
                            e'8 (
                            f'8
                            g'8 )
                        }
                        {
                            \time 4/8
                            fs'4 (
                            e'8
                            d'8 )
                        }
                    }
                >>
            }

        ::

            >>> show(lilypond_file) # doctest: +SKIP

    Makes LilyPond file.

    Wraps `music` in LilyPond ``\score`` block.

    Adds LilyPond ``\header``, ``\layout``, ``\paper`` and ``\score`` blocks to
    LilyPond file.

    Defines layout settings for custom ``\TimeSignatureContext``.

    (Note that you must create and populate an Abjad context with name
    equal to ``'TimeSignatureContext'`` in order for ``\TimeSignatureContext``
    layout settings to apply.)

    Applies many file, layout and paper settings.

    Returns LilyPond file.
    '''
    from abjad.tools import lilypondfiletools

    lilypond_file = lilypondfiletools.LilyPondFile.new(
        music=music,
        default_paper_size=('letter', 'portrait'),
        global_staff_size=16,
        )

    lilypond_file.paper_block.left_margin = 20
    vector = schemetools.make_spacing_vector(0, 0, 12, 0)
    lilypond_file.paper_block.system_system_spacing = vector

    lilypond_file.layout_block.indent = 0
    lilypond_file.layout_block.ragged_right = True
    command = indicatortools.LilyPondCommand('accidentalStyle forget')
    lilypond_file.layout_block.items.append(command)

    block = _make_time_signature_context_block(font_size=1, padding=6)
    lilypond_file.layout_block.items.append(block)

    context_block = lilypondfiletools.ContextBlock(
        source_context_name='Score',
        )
    lilypond_file.layout_block.items.append(context_block)
    context_block.accepts_commands.append('TimeSignatureContext')
    context_block.remove_commands.append('Bar_number_engraver')
    override(context_block).beam.breakable = True
    override(context_block).spacing_spanner.strict_grace_spacing = True
    override(context_block).spacing_spanner.strict_note_spacing = True
    override(context_block).spacing_spanner.uniform_stretching = True
    override(context_block).tuplet_bracket.bracket_visibility = True
    override(context_block).tuplet_bracket.padding = 2
    scheme = schemetools.Scheme('ly:spanner::set-spacing-rods')
    override(context_block).tuplet_bracket.springs_and_rods = scheme
    override(context_block).tuplet_bracket.minimum_length = 3
    scheme = schemetools.Scheme('tuplet-number::calc-fraction-text')
    override(context_block).tuplet_number.text = scheme
    set_(context_block).autoBeaming = False
    moment = schemetools.SchemeMoment((1, 24))
    set_(context_block).proportionalNotationDuration = moment
    set_(context_block).tupletFullLength = True

    # provided as a stub position for user customization
    context_block = lilypondfiletools.ContextBlock(
        source_context_name='StaffGroup',
        )
    lilypond_file.layout_block.items.append(context_block)

    context_block = lilypondfiletools.ContextBlock(
        source_context_name='Staff',
        )

    lilypond_file.layout_block.items.append(context_block)
    context_block.remove_commands.append('Time_signature_engraver')

    context_block = lilypondfiletools.ContextBlock(
        source_context_name='RhythmicStaff',
        )
    lilypond_file.layout_block.items.append(context_block)
    context_block.remove_commands.append('Time_signature_engraver')

    return lilypond_file


def _make_time_signature_context_block(
    font_size=3,
    minimum_distance=10,
    padding=4,
    ):
    from abjad.tools import lilypondfiletools
    assert isinstance(font_size, (int, float))
    assert isinstance(padding, (int, float))
    context_block = lilypondfiletools.ContextBlock(
        type_='Engraver_group',
        name='TimeSignatureContext',
        )
    context_block.consists_commands.append('Axis_group_engraver')
    context_block.consists_commands.append('Time_signature_engraver')
    override(context_block).time_signature.X_extent = (0, 0)
    override(context_block).time_signature.X_offset = schemetools.Scheme(
        'ly:self-alignment-interface::x-aligned-on-self')
    override(context_block).time_signature.Y_extent = (0, 0)
    override(context_block).time_signature.break_align_symbol = False
    override(context_block).time_signature.break_visibility = \
        schemetools.Scheme('end-of-line-invisible')
    override(context_block).time_signature.font_size = font_size
    override(context_block).time_signature.self_alignment_X = \
        schemetools.Scheme('center')
    spacing_vector = schemetools.make_spacing_vector(
        0,
        minimum_distance,
        padding,
        0,
        )
    override(context_block).vertical_axis_group.default_staff_staff_spacing = \
        spacing_vector
    return context_block



# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools.abctools import AbjadValueObject
from abjad.tools.topleveltools import inspect_
from abjad.tools.topleveltools import iterate


class TupletSpellingSpecifier(AbjadValueObject):
    r'''Tuplet spelling specifier.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Specifiers'

    __slots__ = (
        '_avoid_dots',
        '_rewrite_rest_filled_tuplets',
        '_flatten_trivial_tuplets',
        '_is_diminution',
        '_preferred_denominator',
        '_simplify_redundant_tuplets',
        '_use_note_duration_bracket',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        avoid_dots=False,
        flatten_trivial_tuplets=False,
        is_diminution=True,
        preferred_denominator=None,
        rewrite_rest_filled_tuplets=False,
        simplify_redundant_tuplets=False,
        use_note_duration_bracket=False,
        ):
        # TODO: Consider renaming is_diminution=True to is_augmentation=None.
        #       That would allow for all keywords to default to None,
        #       and therefore a single-line storage format.
        self._avoid_dots = bool(avoid_dots)
        self._flatten_trivial_tuplets = bool(flatten_trivial_tuplets)
        self._is_diminution = bool(is_diminution)
        self._preferred_denominator = preferred_denominator
        self._rewrite_rest_filled_tuplets = bool(rewrite_rest_filled_tuplets)
        self._simplify_redundant_tuplets = bool(simplify_redundant_tuplets)
        self._use_note_duration_bracket = bool(use_note_duration_bracket)

    ### PRIVATE METHODS ###

    def _apply_preferred_denominator(self, selections, divisions):
        from abjad.tools import scoretools
        if not self.preferred_denominator:
            return
        tuplets = iterate(selections).by_class(scoretools.Tuplet)
        tuplets = list(tuplets)
        if divisions is None:
            divisions = len(tuplets) * [None]
        assert len(selections) == len(divisions)
        assert len(tuplets) == len(divisions)
        preferred_denominator = self.preferred_denominator
        if isinstance(preferred_denominator, tuple):
            preferred_denominator = durationtools.Duration(
                preferred_denominator,
                )
        for tuplet, division in zip(tuplets, divisions):
            if preferred_denominator == 'divisions':
                tuplet.preferred_denominator = division.numerator
            elif isinstance(preferred_denominator, durationtools.Duration):
                unit_duration = preferred_denominator
                assert unit_duration.numerator == 1
                duration = inspect_(tuplet).get_duration()
                denominator = unit_duration.denominator
                nonreduced_fraction = duration.with_denominator(denominator)
                tuplet.preferred_denominator = nonreduced_fraction.numerator
            elif mathtools.is_positive_integer(preferred_denominator):
                tuplet.preferred_denominator = preferred_denominator
            else:
                message = 'invalid value for preferred denominator: {!r}.'
                message = message.format(preferred_denominator)
                raise Exception(message)

    def _do_simplify_redundant_tuplets(self, selections):
        from abjad.tools import scoretools
        if not self.simplify_redundant_tuplets:
            return
        for tuplet in iterate(selections).by_class(scoretools.Tuplet):
            tuplet._simplify_redundant_tuplet()

    ### PUBLIC PROPERTIES ###

    @property
    def avoid_dots(self):
        r'''Is true when tuplet spelling should avoid dotted rhythmic values.
        Otherwise false.

        Defaults to false.

        Set to true or false.

        Returns true or false.
        '''
        return self._avoid_dots

    @property
    def flatten_trivial_tuplets(self):
        r'''Is true when tuplet spelling should flatten trivial tuplets.
        Otherwise false.

        Defaults to false.

        Set to true or false.

        Returns true or false.
        '''
        return self._flatten_trivial_tuplets

    @property
    def is_diminution(self):
        r'''Is true when tuplet should be spelled as diminution. Otherwise
        false.

        Defaults to true.

        Set to true or false.

        Returns true or false.
        '''
        return self._is_diminution

    @property
    def preferred_denominator(self):
        r'''Gets preferred denominator.

        ..  container:: example

            Tuplet numerators and denominators are reduced to numbers that are
            relatively prime when `preferred_denominator` is set to none. This
            means that ratios like ``6:4`` and ``10:8`` do not arise:

            ::

                >>> rhythm_maker = rhythmmakertools.TupletRhythmMaker(
                ...     tuplet_ratios=[(1, 4)],
                ...     tuplet_spelling_specifier=rhythmmakertools.TupletSpellingSpecifier(
                ...         avoid_dots=True,
                ...         preferred_denominator=None,
                ...         ),
                ...     )

            ::

                >>> divisions = [(2, 16), (4, 16), (6, 16), (8, 16)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 2/16
                        \times 4/5 {
                            c'32 [
                            c'8 ]
                        }
                    }
                    {
                        \time 4/16
                        \times 4/5 {
                            c'16
                            c'4
                        }
                    }
                    {
                        \time 6/16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 3/5 {
                            c'8
                            c'2
                        }
                    }
                    {
                        \time 8/16
                        \times 4/5 {
                            c'8
                            c'2
                        }
                    }
                }

        ..  container:: example

            The preferred denominator of each tuplet is set to the numerator of
            the division that generates the tuplet when `preferred_denominator`
            is set to the string ``'divisions'``. This means that the tuplet
            numerator and denominator are not necessarily relatively prime.
            This also means that ratios like ``6:4`` and ``10:8`` may arise:

            ::

                >>> rhythm_maker = rhythmmakertools.TupletRhythmMaker(
                ...     tuplet_ratios=[(1, 4)],
                ...     tuplet_spelling_specifier=rhythmmakertools.TupletSpellingSpecifier(
                ...         avoid_dots=True,
                ...         preferred_denominator='divisions',
                ...         ),
                ...     )

            ::

                >>> divisions = [(2, 16), (4, 16), (6, 16), (8, 16)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 2/16
                        \times 4/5 {
                            c'32 [
                            c'8 ]
                        }
                    }
                    {
                        \time 4/16
                        \times 4/5 {
                            c'16
                            c'4
                        }
                    }
                    {
                        \time 6/16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 6/10 {
                            c'8
                            c'2
                        }
                    }
                    {
                        \time 8/16
                        \times 8/10 {
                            c'8
                            c'2
                        }
                    }
                }

        ..  container:: example

            The preferred denominator of each tuplet is set in terms of a unit
            duration when `preferred_denominator` is set to a duration. The
            setting does not affect the first tuplet:

            ::

                >>> rhythm_maker = rhythmmakertools.TupletRhythmMaker(
                ...     tuplet_ratios=[(1, 4)],
                ...     tuplet_spelling_specifier=rhythmmakertools.TupletSpellingSpecifier(
                ...         avoid_dots=True,
                ...         preferred_denominator=Duration(1, 16),
                ...         ),
                ...     )

            ::

                >>> divisions = [(2, 16), (4, 16), (6, 16), (8, 16)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 2/16
                        \times 4/5 {
                            c'32 [
                            c'8 ]
                        }
                    }
                    {
                        \time 4/16
                        \times 4/5 {
                            c'16
                            c'4
                        }
                    }
                    {
                        \time 6/16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 6/10 {
                            c'8
                            c'2
                        }
                    }
                    {
                        \time 8/16
                        \times 8/10 {
                            c'8
                            c'2
                        }
                    }
                }

        ..  container:: example

            Sets the preferred denominator of each tuplet in terms 32nd notes.
            The setting affects all tuplets:

            ::

                >>> rhythm_maker = rhythmmakertools.TupletRhythmMaker(
                ...     tuplet_ratios=[(1, 4)],
                ...     tuplet_spelling_specifier=rhythmmakertools.TupletSpellingSpecifier(
                ...         avoid_dots=True,
                ...         preferred_denominator=Duration(1, 32),
                ...         ),
                ...     )

            ::

                >>> divisions = [(2, 16), (4, 16), (6, 16), (8, 16)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 2/16
                        \times 4/5 {
                            c'32 [
                            c'8 ]
                        }
                    }
                    {
                        \time 4/16
                        \times 8/10 {
                            c'16
                            c'4
                        }
                    }
                    {
                        \time 6/16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 12/20 {
                            c'8
                            c'2
                        }
                    }
                    {
                        \time 8/16
                        \times 16/20 {
                            c'8
                            c'2
                        }
                    }
                }

        ..  container:: example

            Sets the preferred denominator each tuplet in terms 64th notes. The
            setting affects all tuplets:

            ::

                >>> rhythm_maker = rhythmmakertools.TupletRhythmMaker(
                ...     tuplet_ratios=[(1, 4)],
                ...     tuplet_spelling_specifier=rhythmmakertools.TupletSpellingSpecifier(
                ...         avoid_dots=True,
                ...         preferred_denominator=Duration(1, 64),
                ...         ),
                ...     )

            ::

                >>> divisions = [(2, 16), (4, 16), (6, 16), (8, 16)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 2/16
                        \times 8/10 {
                            c'32 [
                            c'8 ]
                        }
                    }
                    {
                        \time 4/16
                        \times 16/20 {
                            c'16
                            c'4
                        }
                    }
                    {
                        \time 6/16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 24/40 {
                            c'8
                            c'2
                        }
                    }
                    {
                        \time 8/16
                        \times 32/40 {
                            c'8
                            c'2
                        }
                    }
                }

        ..  container:: example

            The preferred denominator of each tuplet is set directly when
            `preferred_denominator` is set to a positive integer. This example
            sets the preferred denominator of each tuplet to ``8``. Setting
            does not affect the third tuplet:

            ::

                >>> rhythm_maker = rhythmmakertools.TupletRhythmMaker(
                ...     tuplet_ratios=[(1, 4)],
                ...     tuplet_spelling_specifier=rhythmmakertools.TupletSpellingSpecifier(
                ...         avoid_dots=True,
                ...         preferred_denominator=8,
                ...         ),
                ...     )

            ::

                >>> divisions = [(2, 16), (4, 16), (6, 16), (8, 16)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 2/16
                        \times 8/10 {
                            c'32 [
                            c'8 ]
                        }
                    }
                    {
                        \time 4/16
                        \times 8/10 {
                            c'16
                            c'4
                        }
                    }
                    {
                        \time 6/16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 3/5 {
                            c'8
                            c'2
                        }
                    }
                    {
                        \time 8/16
                        \times 8/10 {
                            c'8
                            c'2
                        }
                    }
                }

        ..  container:: example

            Sets the preferred denominator of each tuplet to ``12``. Setting
            affects all tuplets:

            ::

                >>> rhythm_maker = rhythmmakertools.TupletRhythmMaker(
                ...     tuplet_ratios=[(1, 4)],
                ...     tuplet_spelling_specifier=rhythmmakertools.TupletSpellingSpecifier(
                ...         avoid_dots=True,
                ...         preferred_denominator=12,
                ...         ),
                ...     )

            ::

                >>> divisions = [(2, 16), (4, 16), (6, 16), (8, 16)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 2/16
                        \times 12/15 {
                            c'32 [
                            c'8 ]
                        }
                    }
                    {
                        \time 4/16
                        \times 12/15 {
                            c'16
                            c'4
                        }
                    }
                    {
                        \time 6/16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 12/20 {
                            c'8
                            c'2
                        }
                    }
                    {
                        \time 8/16
                        \times 12/15 {
                            c'8
                            c'2
                        }
                    }
                }

        ..  container:: example

            Sets the preferred denominator of each tuplet to ``13``. Setting
            does not affect any tuplet:

            ::

                >>> rhythm_maker = rhythmmakertools.TupletRhythmMaker(
                ...     tuplet_ratios=[(1, 4)],
                ...     tuplet_spelling_specifier=rhythmmakertools.TupletSpellingSpecifier(
                ...         avoid_dots=True,
                ...         preferred_denominator=13,
                ...         ),
                ...     )

            ::

                >>> divisions = [(2, 16), (4, 16), (6, 16), (8, 16)]
                >>> selections = rhythm_maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     selections,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> f(lilypond_file[Staff])
                \new RhythmicStaff {
                    {
                        \time 2/16
                        \times 4/5 {
                            c'32 [
                            c'8 ]
                        }
                    }
                    {
                        \time 4/16
                        \times 4/5 {
                            c'16
                            c'4
                        }
                    }
                    {
                        \time 6/16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 3/5 {
                            c'8
                            c'2
                        }
                    }
                    {
                        \time 8/16
                        \times 4/5 {
                            c'8
                            c'2
                        }
                    }
                }

        Defaults to none.

        Set to ``'divisions'``, duration, positive integer or none.

        Returns ``'divisions'``, duration, positive integer or none.
        '''
        return self._preferred_denominator

    @property
    def rewrite_rest_filled_tuplets(self):
        r'''Is true when tuplet spelling should flatten rest-filled tuplets.
        Otherwise false.

        Defaults to false.

        Set to true or false.

        Returns true or false.
        '''
        return self._rewrite_rest_filled_tuplets

    @property
    def simplify_redundant_tuplets(self):
        r'''Is true when tuplets should be simplified. Otherwise false.

        Defaults to false.

        Set to true or false

        Returns true or false.
        '''
        return self._simplify_redundant_tuplets

    @property
    def use_note_duration_bracket(self):
        r'''Is true when tuplet should override tuplet number text with note
        duration bracket giving tuplet duration. Otherwise false.

        Defaults to false.

        Set to true or false.

        Returns true or false.
        '''
        return self._use_note_duration_bracket



# -*- coding: utf-8 -*-


def is_positive_integer_power_of_two(argument):
    r'''Is true when `argument` is a positive integer power of 2. Otherwise
    false.

    ..  container:: example

        ::

            >>> for n in range(10):
            ...     print(n, mathtools.is_positive_integer_power_of_two(n))
            ... 
            0 False
            1 True
            2 True
            3 False
            4 True
            5 False
            6 False
            7 False
            8 True
            9 False

    Returns true or false.
    '''
    from abjad.tools import mathtools
    return (
        0 < argument and
        mathtools.is_nonnegative_integer_power_of_two(argument)
        )



# -*- coding: utf-8 -*-
from abjad.tools.documentationtools.ReSTDirective import ReSTDirective


class ReSTGraphvizDirective(ReSTDirective):
    r'''A ReST Graphviz directive.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'reStructuredText'

    ### INITIALIZER ###

    def __init__(
        self,
        graph=None,
        name=None,
        options=None,
        ):
        from abjad.tools import graphtools
        if graph is not None:
            assert isinstance(graph, graphtools.GraphvizGraph)
        self._graph = graph
        ReSTDirective.__init__(
            self,
            name=name,
            options=options,
            )

    ### PRIVATE PROPERTIES ###

    @property
    def _children_rest_format_contributions(self):
        result = ['']
        graphviz_format = str(self.graph)
        for line in graphviz_format.splitlines():
            line = '   {}'.format(line)
            result.append(line)
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def directive(self):
        r'''Gets directive of ReST Graphviz directive.

        Returns string.
        '''
        return 'graphviz'

    @property
    def graph(self):
        r'''Gets graph of ReST Graphviz directive.

        Returns Graphviz graph.
        '''
        return self._graph



# -*- coding: utf-8 -*-
from abjad import *



def test_scoretools_Container___delitem___01():
    r'''Deletes in-score container.
    '''

    voice = Voice("{ c'8 ( d'8 ) } { e'8 ( f'8 ) }")
    leaves = select(voice).by_leaf()
    attach(Beam(), leaves)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [ (
                d'8 )
            }
            {
                e'8 (
                f'8 ] )
            }
        }
        '''
        )

    container = voice[0]
    del(voice[0])

    # container no longer appears in score
    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                e'8 [ (
                f'8 ] )
            }
        }
        '''
        )

    assert inspect_(voice).is_well_formed()

    # container leaves are still slurred
    assert format(container) == stringtools.normalize(
        r'''
        {
            c'8 (
            d'8 )
        }
        '''
        )

    assert inspect_(container).is_well_formed()


def test_scoretools_Container___delitem___02():
    r'''Deletes in-score leaf.
    '''

    voice = Voice("c'8 [ d'8 e'8 f'8 ]")
    del(voice[1])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            e'8
            f'8 ]
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_scoretools_Container___delitem___03():
    r'''Deletes slice in middle of container.
    '''

    voice = Voice("c'8 [ d'8 e'8 f'8 ]")
    del(voice[1:3])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            f'8 ]
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_scoretools_Container___delitem___04():
    r'''Delete slice at beginning of container.
    '''

    voice = Voice("c'8 [ d'8 e'8 f'8 ]")
    del(voice[:2])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            e'8 [
            f'8 ]
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_scoretools_Container___delitem___05():
    r'''Deletes slice at end of container.
    '''

    voice = Voice("c'8 [ d'8 e'8 f'8 ]")
    del(voice[2:])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            d'8 ]
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_scoretools_Container___delitem___06():
    r'''Deletes container contents.
    '''

    voice = Voice("c'8 [ d'8 e'8 f'8 ]")
    del(voice[:])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
        }
        '''
        )

    assert not len(voice)


def test_scoretools_Container___delitem___07():
    r'''Deletes leaf from tuplet.
    '''

    tuplet = Tuplet(Multiplier((2, 3)), "c'8 [ d'8 e'8 ]")
    del(tuplet[1])

    assert format(tuplet) == stringtools.normalize(
        r'''
        \tweak edge-height #'(0.7 . 0)
        \times 2/3 {
            c'8 [
            e'8 ]
        }
        '''
        )

    assert inspect_(tuplet).is_well_formed()


def test_scoretools_Container___delitem___08():
    r'''Deletes leaf from nested container.
    '''

    voice = Voice("c'8 [ { d'8 e'8 } f'8 ]")
    leaves = select(voice).by_leaf()
    attach(Glissando(), list(leaves))

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [ \glissando
            {
                d'8 \glissando
                e'8 \glissando
            }
            f'8 ]
        }
        '''
        )

    leaf = leaves[1]
    del(voice[1][0])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [ \glissando
            {
                e'8 \glissando
            }
            f'8 ]
        }
        '''
        )

    assert inspect_(voice).is_well_formed()
    assert inspect_(leaf).is_well_formed()



# -*- coding: utf-8 -*-
from abjad.tools import scoretools
from abjad.tools import selectiontools
from abjad.tools import spannertools
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import mutate
from abjad.tools.topleveltools import select


def edit_first_violin_voice(score, durated_reservoir):
    r'''Edits first violin voice.
    '''

    voice = score['First Violin Voice']
    descents = durated_reservoir['First Violin']
    descents = selectiontools.Selection(descents)

    last_descent = select(descents[-1])
    copied_descent = mutate(last_descent).copy()
    voice.extend(copied_descent)

    final_sustain_rhythm = [(6, 4)] * 43 + [(1, 2)]
    final_sustain_notes = scoretools.make_notes(["c'"], final_sustain_rhythm)
    voice.extend(final_sustain_notes)
    tie = spannertools.Tie()
    attach(tie, final_sustain_notes)
    voice.extend('r4 r2.')


# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_MeasuredComplexBeam___init___01():
    r'''Initialize empty measured complex beam spanner.
    '''

    beam = spannertools.MeasuredComplexBeam()
    assert isinstance(beam, spannertools.MeasuredComplexBeam)



# -*- coding: utf-8 -*-
import pytest
from abjad import *
from abjad.tools.lilypondparsertools import LilyPondParser


def test_lilypondparsertools_LilyPondParser__spanners__Tie_01():

    target = Container([Note(0, 1), Note(0, 1)])
    tie = spannertools.Tie()
    attach(tie, target[:])
    parser = LilyPondParser()
    result = parser(format(target))
    assert format(target) == format(result) and target is not result


def test_lilypondparsertools_LilyPondParser__spanners__Tie_02():

    string = r'{ c ~ }'
    assert pytest.raises(Exception, 'LilyPondParser()(string)')


def test_lilypondparsertools_LilyPondParser__spanners__Tie_03():

    string = r'{ ~ c }'
    assert pytest.raises(Exception, 'LilyPondParser()(string)')


def test_lilypondparsertools_LilyPondParser__spanners__Tie_04():
    r'''With direction.
    '''

    target = Container([Note(0, 1), Note(0, 1)])
    tie = spannertools.Tie(direction=Up)
    attach(tie, target[:])
    parser = LilyPondParser()
    result = parser(format(target))
    assert format(target) == format(result) and target is not result


def test_lilypondparsertools_LilyPondParser__spanners__Tie_05():
    r'''With direction.
    '''

    target = Container([Note(0, 1), Note(0, 1)])
    tie = spannertools.Tie(direction=Down)
    attach(tie, target[:])
    parser = LilyPondParser()
    result = parser(format(target))
    assert format(target) == format(result) and target is not result



# -*- coding: utf-8 -*-
import math
import numbers


def greatest_power_of_two_less_equal(n, i=0):
    r'''Gets greatest integer power of two less than or equal to positive `n`.

    ..  container:: example

        ::

            >>> for n in range(10, 20):
            ...     print('\t%s\t%s' % (n, mathtools.greatest_power_of_two_less_equal(n)))
            ... 
                10 8
                11 8
                12 8
                13 8
                14 8
                15 8
                16 16
                17 16
                18 16
                19 16

        Greatest-but-``i`` integer power of ``2`` less than or equal to
        positive `n`:

        ::

            >>> for n in range(10, 20):
            ...     print('\t%s\t%s' % (n, mathtools.greatest_power_of_two_less_equal(n, i=1)))
            ... 
                10 4
                11 4
                12 4
                13 4
                14 4
                15 4
                16 8
                17 8
                18 8
                19 8

    Returns positive integer.
    '''
    if not isinstance(n, numbers.Number):
        message = 'must be number: {!r}.'
        message = message.format(n)
        raise TypeError(message)
    if n <= 0:
        message = 'must be positive: {!r}.'
        message = message.format(n)
        raise ValueError(message)
    return 2 ** (int(math.log(n, 2)) - i)



# -*- coding: utf-8 -*-


def override(argument):
    r'''Makes LilyPond grob name manager.

    ..  container:: example

        Overrides staff symbol color:

        ::

            >>> staff = Staff("c'4 e'4 d'4 f'4")
            >>> override(staff).staff_symbol.color = 'red'
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff \with {
                \override StaffSymbol.color = #red
            } {
                c'4
                e'4
                d'4
                f'4
            }

    ..  container:: example

        Returns LilyPond grob name manager:

        ::

            >>> override(staff)
            LilyPondGrobNameManager(('staff_symbol', LilyPondNameManager(('color', 'red'))))

    '''
    from abjad.tools import lilypondnametools
    if getattr(argument, '_lilypond_grob_name_manager', None) is None:
        manager = lilypondnametools.LilyPondGrobNameManager()
        argument._lilypond_grob_name_manager = manager
    return argument._lilypond_grob_name_manager



# -*- coding: utf-8 -*-
from abjad import *


def test_selectiontools_LogicalTie_written_duration_01():

    staff = Staff("c' ~ c'16")

    assert inspect_(staff[0]).get_logical_tie().written_duration == Duration(5, 16)


def test_selectiontools_LogicalTie_written_duration_02():

    staff = Staff("c'")

    assert inspect_(staff[0]).get_logical_tie().written_duration == Duration(1, 4)



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NamedIntervalClass_from_pitch_carriers_01():
    r'''Ascending intervals greater than an octave.
    '''

    mcpi = pitchtools.NamedIntervalClass.from_pitch_carriers(
        NamedPitch(-3), NamedPitch(12))
    assert mcpi == pitchtools.NamedIntervalClass('minor', 3)

    mcpi = pitchtools.NamedIntervalClass.from_pitch_carriers(
        NamedPitch(-2), NamedPitch(12))
    assert mcpi == pitchtools.NamedIntervalClass('major', 2)

    mcpi = pitchtools.NamedIntervalClass.from_pitch_carriers(
        NamedPitch(-1), NamedPitch(12))
    assert mcpi == pitchtools.NamedIntervalClass('minor', 2)


def test_pitchtools_NamedIntervalClass_from_pitch_carriers_02():
    r'''Ascending octave.
    '''

    mcpi = pitchtools.NamedIntervalClass.from_pitch_carriers(
        NamedPitch(0), NamedPitch(12))
    assert mcpi == pitchtools.NamedIntervalClass('perfect', 8)


def test_pitchtools_NamedIntervalClass_from_pitch_carriers_03():
    r'''Ascending intervals less than an octave.
    '''

    mcpi = pitchtools.NamedIntervalClass.from_pitch_carriers(
        NamedPitch(9), NamedPitch(12))
    assert mcpi == pitchtools.NamedIntervalClass('minor', 3)

    mcpi = pitchtools.NamedIntervalClass.from_pitch_carriers(
        NamedPitch(10), NamedPitch(12))
    assert mcpi == pitchtools.NamedIntervalClass('major', 2)

    mcpi = pitchtools.NamedIntervalClass.from_pitch_carriers(
        NamedPitch(11), NamedPitch(12))
    assert mcpi == pitchtools.NamedIntervalClass('minor', 2)


def test_pitchtools_NamedIntervalClass_from_pitch_carriers_04():
    r'''Unison.
    '''

    mcpi = pitchtools.NamedIntervalClass.from_pitch_carriers(
        NamedPitch(12), NamedPitch(12))
    assert mcpi == pitchtools.NamedIntervalClass('perfect', 1)


def test_pitchtools_NamedIntervalClass_from_pitch_carriers_05():
    r'''Descending intervals greater than an octave.
    '''

    mcpi = pitchtools.NamedIntervalClass.from_pitch_carriers(
        NamedPitch(12), NamedPitch(-3))
    assert mcpi == pitchtools.NamedIntervalClass('minor', -3)

    mcpi = pitchtools.NamedIntervalClass.from_pitch_carriers(
        NamedPitch(12), NamedPitch(-2))
    assert mcpi == pitchtools.NamedIntervalClass('major', -2)

    mcpi = pitchtools.NamedIntervalClass.from_pitch_carriers(
        NamedPitch(12), NamedPitch(-1))
    assert mcpi == pitchtools.NamedIntervalClass('minor', -2)


def test_pitchtools_NamedIntervalClass_from_pitch_carriers_06():
    r'''Descending octave.
    '''

    mcpi = pitchtools.NamedIntervalClass.from_pitch_carriers(
        NamedPitch(12), NamedPitch(0))
    assert mcpi == pitchtools.NamedIntervalClass('perfect', -8)


def test_pitchtools_NamedIntervalClass_from_pitch_carriers_07():
    r'''Descending intervals less than an octave.
    '''

    mcpi = pitchtools.NamedIntervalClass.from_pitch_carriers(
        NamedPitch(12), NamedPitch(9))
    assert mcpi == pitchtools.NamedIntervalClass('minor', -3)

    mcpi = pitchtools.NamedIntervalClass.from_pitch_carriers(
        NamedPitch(12), NamedPitch(10))
    assert mcpi == pitchtools.NamedIntervalClass('major', -2)

    mcpi = pitchtools.NamedIntervalClass.from_pitch_carriers(
        NamedPitch(12), NamedPitch(11))
    assert mcpi == pitchtools.NamedIntervalClass('minor', -2)



# -*- coding: utf-8 -*-
import os
import platform
import shutil
import unittest
from abjad.tools import abjadbooktools
from abjad.tools import stringtools


@unittest.skipIf(
    platform.python_implementation() != 'CPython',
    'Only for CPython.',
    )
class TestLaTeXDocumentHandler(unittest.TestCase):

    maxDiff = None

    test_directory = os.path.dirname(__file__)
    assets_directory = os.path.join(test_directory, 'assets')
    source_path = os.path.join(
        test_directory,
        'chapters',
        'chapter-1',
        'source.tex',
        )
    with open(source_path, 'r') as file_pointer:
        source_contents = file_pointer.read()
    target_path = os.path.join(
        test_directory,
        'chapters',
        'chapter-1',
        'target.tex',
        )
    expected_path = os.path.join(
        test_directory,
        'chapters',
        'chapter-1',
        'expected.tex',
        )
    with open(expected_path, 'r') as file_pointer:
        expected_contents = file_pointer.read()
    expected_asset_names = (
        'lilypond-6737d707e144fd1f0af98dd8007ebb4b.ly',
        'lilypond-6737d707e144fd1f0af98dd8007ebb4b.pdf',
        )

    def setUp(self):
        if os.path.exists(self.assets_directory):
            shutil.rmtree(self.assets_directory)
        if os.path.exists(self.target_path):
            os.remove(self.target_path)

    def tearDown(self):
        if os.path.exists(self.assets_directory):
            shutil.rmtree(self.assets_directory)
        if os.path.exists(self.target_path):
            os.remove(self.target_path)
        with open(self.source_path, 'w') as file_pointer:
            file_pointer.write(self.source_contents)

    def test_latex_root_directory_1(self):
        input_file_contents = [
            '\\begin{comment}',
            '<abjad>',
            'note = Note(0, (1, 4))',
            'show(note)',
            '</abjad>',
            '\\end{comment}',
            ]
        assets_directory = 'ExamplePaper/assets'
        input_file_path = 'ExamplePaper/chapters/chapter-1/section-2.tex'
        latex_root_directory = 'ExamplePaper'
        document_handler = abjadbooktools.LaTeXDocumentHandler(
            assets_directory=assets_directory,
            input_file_contents=input_file_contents,
            input_file_path=input_file_path,
            latex_root_directory=latex_root_directory,
            )
        rebuilt_source = document_handler(return_source=True)
        assert rebuilt_source == stringtools.normalize(
            '''
            \\begin{comment}
            <abjad>
            note = Note(0, (1, 4))
            show(note)
            </abjad>
            \\end{comment}

            %%% ABJADBOOK START %%%
            \\begin{lstlisting}
            >>> note = Note(0, (1, 4))
            >>> show(note)
            \\end{lstlisting}
            \\noindent\\includegraphics{assets/lilypond-6737d707e144fd1f0af98dd8007ebb4b.pdf}
            %%% ABJADBOOK END %%%
            ''',
            )

    def test_latex_root_directory_2(self):
        assert not os.path.exists(self.target_path)
        assert not os.path.exists(self.assets_directory)
        document_handler = abjadbooktools.LaTeXDocumentHandler.from_path(
            input_file_path=self.source_path,
            assets_directory=self.assets_directory,
            latex_root_directory=self.test_directory,
            )
        document_handler(output_file_path=self.target_path)
        assert os.path.exists(self.target_path)
        assert os.path.exists(self.assets_directory)
        with open(self.target_path, 'r') as file_pointer:
            target_contents = file_pointer.read()
        assert stringtools.normalize(target_contents) == \
            stringtools.normalize(self.expected_contents)
        assert tuple(sorted(os.listdir(self.assets_directory))) == \
            self.expected_asset_names



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools import tonalanalysistools


def test_tonalanalysistools_RomanNumeral_bass_scale_degree_01():

    roman_numeral = tonalanalysistools.RomanNumeral(5, 'major', 5, 0)
    assert roman_numeral.bass_scale_degree == tonalanalysistools.ScaleDegree(5)

    roman_numeral = tonalanalysistools.RomanNumeral(5, 'major', 5, 1)
    assert roman_numeral.bass_scale_degree == tonalanalysistools.ScaleDegree(7)

    roman_numeral = tonalanalysistools.RomanNumeral(5, 'major', 5, 2)
    assert roman_numeral.bass_scale_degree == tonalanalysistools.ScaleDegree(2)


def test_tonalanalysistools_RomanNumeral_bass_scale_degree_02():

    roman_numeral = tonalanalysistools.RomanNumeral(5, 'major', 7, 0)
    assert roman_numeral.bass_scale_degree == tonalanalysistools.ScaleDegree(5)

    roman_numeral = tonalanalysistools.RomanNumeral(5, 'major', 7, 1)
    assert roman_numeral.bass_scale_degree == tonalanalysistools.ScaleDegree(7)

    roman_numeral = tonalanalysistools.RomanNumeral(5, 'major', 7, 2)
    assert roman_numeral.bass_scale_degree == tonalanalysistools.ScaleDegree(2)

    roman_numeral = tonalanalysistools.RomanNumeral(5, 'major', 7, 3)
    assert roman_numeral.bass_scale_degree == tonalanalysistools.ScaleDegree(4)



# -*- coding: utf-8 -*-


def make_piano_score_from_leaves(leaves, lowest_treble_pitch=None):
    r"""Make piano score from `leaves`:

    ::

        >>> notes = [Note(x, (1, 4)) for x in [-12, 37, -10, 2, 4, 17]]
        >>> score, treble_staff, bass_staff = scoretools.make_piano_score_from_leaves(notes)

    ..  doctest::

        >>> print(format(score))
        \new Score <<
            \new PianoStaff <<
                \context Staff = "treble" {
                    \clef "treble"
                    r4
                    cs''''4
                    r4
                    d'4
                    e'4
                    f''4
                }
                \context Staff = "bass" {
                    \clef "bass"
                    c4
                    r4
                    d4
                    r4
                    r4
                    r4
                }
            >>
        >>

    ::

        >>> show(score) # doctest: +SKIP

    When ``lowest_treble_pitch=None`` set to B3.

    Returns score, treble staff, bass staff.
    """
    from abjad.tools import pitchtools
    from abjad.tools import scoretools

    if lowest_treble_pitch is None:
        lowest_treble_pitch = pitchtools.NamedPitch('b')

    score, treble_staff, bass_staff = scoretools.make_empty_piano_score()
    for leaf in leaves:
        treble_chord, bass_chord = leaf._divide(lowest_treble_pitch)
        treble_staff.append(treble_chord)
        bass_staff.append(bass_chord)

    return score, treble_staff, bass_staff



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_datastructuretools_TreeContainer_name_01():

    foo = datastructuretools.TreeContainer(name='foo')
    bar = datastructuretools.TreeContainer(name='bar')
    baz = datastructuretools.TreeContainer(name='baz')
    quux = datastructuretools.TreeContainer(name='quux')
    quux2 = datastructuretools.TreeContainer(name='quux')

    foo.append(bar)
    bar.extend([baz, quux])
    baz.append(quux2)

    assert format(foo, 'lilypond') == stringtools.normalize(
        r'''
        TreeContainer(
            children=(
                TreeContainer(
                    children=(
                        TreeContainer(
                            children=(
                                TreeContainer(
                                    name='quux'
                                    ),
                                ),
                            name='baz'
                            ),
                        TreeContainer(
                            name='quux'
                            ),
                        ),
                    name='bar'
                    ),
                ),
            name='foo'
            )
        '''
        )

    assert foo['bar'] is bar
    assert foo['baz'] is baz
    assert pytest.raises(ValueError, "foo['quux']")

    quux2.name = 'wux'

    assert foo['quux'] is quux
    assert foo['wux'] is quux2



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_Hairpin_include_rests_01():
    r'''Hairpin spanner avoids rests.
    '''

    staff = Staff(Rest((1, 8)) * 4 + [Note(n, (1, 8)) for n in range(4, 8)])
    crescendo = Crescendo(include_rests=False)
    attach(crescendo, staff[:])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            r8
            r8
            r8
            r8
            e'8 \<
            f'8
            fs'8
            g'8 \!
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_spannertools_Hairpin_include_rests_02():
    r'''Hairpin spanner avoids rests.
    '''

    staff = Staff([Note(n, (1, 8)) for n in range(4)] + Rest((1, 8)) * 4)
    crescendo = Crescendo(include_rests=False)
    attach(crescendo, staff[:])


    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 \<
            cs'8
            d'8
            ef'8 \!
            r8
            r8
            r8
            r8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Skip___init___01():
    r'''Initialize skip from LilyPond input string.
    '''

    skip = scoretools.Skip('s8.')
    assert isinstance(skip, scoretools.Skip)


def test_scoretools_Skip___init___02():
    r'''Initialize skip from containerize note.
    '''

    c = Chord([2, 3, 4], (1, 4))
    duration = c.written_duration
    skip = scoretools.Skip(c)
    assert isinstance(skip, scoretools.Skip)
    # check that attributes have not been removed or added.
    assert dir(c) == dir(Chord([2, 3, 4], (1, 4)))
    assert dir(skip) == dir(scoretools.Skip((1, 4)))
    assert skip._parent is None
    assert skip.written_duration == duration


def test_scoretools_Skip___init___03():
    r'''Initialize skip from tupletized note.
    '''

    tuplet = scoretools.FixedDurationTuplet(
        Duration(2, 8), Chord([2, 3, 4], (1, 4)) * 3)
    d = tuplet[0].written_duration
    skip = scoretools.Skip(tuplet[0])
    assert isinstance(tuplet[0], Chord)
    assert isinstance(skip, scoretools.Skip)
    assert tuplet[0]._parent is tuplet
    assert tuplet[0].written_duration == d
    assert skip._parent is None


def test_scoretools_Skip___init___04():
    r'''Initialize skip from beamed chord.
    '''

    staff = Staff(Chord([2, 3, 4], (1, 4)) * 3)
    beam = Beam()
    attach(beam, staff[:])
    skip = scoretools.Skip(staff[0])
    assert isinstance(staff[0], Chord)
    assert isinstance(skip, scoretools.Skip)
    assert staff[0]._parent is staff
    assert skip._parent is None


def test_scoretools_Skip___init___05():

    note = Note(2, (1, 8))
    d = note.written_duration
    skip = scoretools.Skip(note)
    assert isinstance(skip, scoretools.Skip)
    # check that attributes have not been removed or added.
    assert dir(note) == dir(Note("c'4"))
    assert dir(skip) == dir(scoretools.Skip((1, 4)))
    assert format(skip) == 's8'
    assert skip._parent is None
    assert skip.written_duration == d


def test_scoretools_Skip___init___06():

    tuplet = scoretools.FixedDurationTuplet(
        Duration(2, 8), Note(0, (1, 8)) * 3)
    d = tuplet[0].written_duration
    skip = scoretools.Skip(tuplet[0])
    assert isinstance(tuplet[0], Note)
    assert isinstance(skip, scoretools.Skip)
    assert tuplet[0]._parent is tuplet
    assert tuplet[0].written_duration == d


def test_scoretools_Skip___init___07():
    r'''Initialize skip from beamed note.
    '''

    staff = Staff(Note(0, (1, 8)) * 3)
    beam = Beam()
    attach(beam, staff[:])
    skip = scoretools.Skip(staff[0])
    assert isinstance(staff[0], Note)
    assert isinstance(skip, scoretools.Skip)
    assert staff[0]._parent is staff


def test_scoretools_Skip___init___08():
    r'''Initialize skip from unincorporaed rest.
    '''

    rest = Rest((1, 8))
    d = rest.written_duration
    skip = scoretools.Skip(rest)
    assert isinstance(skip, scoretools.Skip)
    # check that attributes have not been removed or added.
    assert dir(rest) == dir(Rest((1, 4)))
    assert dir(skip) == dir(scoretools.Skip((1, 4)))
    assert skip._parent is None
    assert skip.written_duration == d


def test_scoretools_Skip___init___09():
    r'''Initialize skip from tupletized rest.
    '''

    tuplet = scoretools.FixedDurationTuplet(Duration(2, 8), Rest((1, 8)) * 3)
    d = tuplet[0].written_duration
    skip = scoretools.Skip(tuplet[0])
    assert isinstance(skip, scoretools.Skip)
    assert isinstance(tuplet[0], Rest)
    assert tuplet[0]._parent is tuplet
    assert tuplet[0].written_duration == d
    assert skip._parent is None


def test_scoretools_Skip___init___10():
    r'''Initialize skip from spanned rest.
    '''

    staff = Staff([Note(0, (1, 8)), Rest((1, 8)), Note(0, (1, 8))])
    beam = Beam()
    attach(beam, staff[:])
    skip = scoretools.Skip(staff[1])
    assert isinstance(skip, scoretools.Skip)
    assert isinstance(staff[1], Rest)
    assert staff[1]._parent is staff
    assert skip._parent is None



# -*- coding: utf-8 -*-
from abjad.tools import indicatortools
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import detach


# TODO: implement scoretools.set_measure_denominator_and_adjust_contents().
def set_measure_denominator_and_adjust_numerator(measure, denominator):
    r'''Set `measure` time signature `denominator` and multiply time signature numerator accordingly:

    ::

        >>> measure = Measure((3, 8), "c'8 d'8 e'8")
        >>> beam = spannertools.Beam()
        >>> attach(beam, measure[:])

    ..  doctest::

        >>> print(format(measure))
        {
            \time 3/8
            c'8 [
            d'8
            e'8 ]
        }

    ::

        >>> scoretools.set_measure_denominator_and_adjust_numerator(measure, 16)
        Measure((6, 16), "c'8 d'8 e'8")

    ..  doctest::

        >>> print(format(measure))
        {
            \time 6/16
            c'8 [
            d'8
            e'8 ]
        }

    Leave `measure` contents unchanged.

    Return `measure`.
    '''
    from abjad.tools import scoretools

    if isinstance(measure, scoretools.Measure):
        # to allow iteration inside zero-update loop
        old_time_signature = measure.time_signature
        old_time_signature_pair = (
            old_time_signature.numerator, old_time_signature.denominator)
        new_time_signature = mathtools.NonreducedFraction(old_time_signature_pair)
        new_time_signature = new_time_signature.with_denominator(denominator)
        new_time_signature = indicatortools.TimeSignature(new_time_signature)
        detach(indicatortools.TimeSignature, measure)
        attach(new_time_signature, measure)

    return measure


# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools import systemtools


def test_systemtools_IOManager_get_last_output_file_name_01():

    last_output_file_name = systemtools.IOManager.get_last_output_file_name()

    assert isinstance(last_output_file_name, (str, type(None)))
    if isinstance(last_output_file_name, str):
        if last_output_file_name.endswith('.ly'):
            assert len(last_output_file_name) == 7
        elif last_output_file_name.endswith('.pdf'):
            assert len(last_output_file_name) == 8
        elif last_output_file_name.endswith('.mid'):
            assert len(last_output_file_name) == 8
        elif last_output_file_name.endswith('.midi'):
            assert len(last_output_file_name) == 9
        else:
            message = 'can not pasre last output file name: {!r}.'
            message = message.format(last_output_file_name)
            raise Exception(message)
